#!/usr/bin/env python
"""
_TestSpec_

Test spec with known output modules used for testing.
"""




from WMCore.WMSpec.WMWorkload import newWorkload

class TestWorkloadFactory(object):
    """
    _TestWorkloadFactory_

    Stamp out test workfloads.
    """
    def createWorkload(self):
        """
        _createWorkload_

        Create a new workload.
        """
        workload = newWorkload("TestWorkload")
        workload.setOwner("sfoulkes@fnal.gov")
        workload.setStartPolicy("DatasetBlock", SliceType = "NumberOfFiles", SliceSize = 1)
        workload.setEndPolicy("SingleShot")
        workload.setAcquisitionEra("WMAgentCommissioning10")
        return workload

    def setupProcessingTask(self, procTask):
        """
        _setupProcessingTask_

        Given an empty task add cmsRun, stagOut and logArch steps.
        """
        procTaskCmssw = procTask.makeStep("cmsRun1")
        procTaskCmssw.setStepType("CMSSW")
        procTaskStageOut = procTaskCmssw.addStep("stageOut1")
        procTaskStageOut.setStepType("StageOut")
        procTaskLogArch = procTaskCmssw.addStep("logArch1")
        procTaskLogArch.setStepType("LogArchive")
        procTask.applyTemplates()
        splitArgs = {"files_per_job": 1}
        procTask.setSplittingAlgorithm("FileBased", **splitArgs)
        procTask.setTaskType("Processing")

        procTask.addInputDataset(name="/MinimumBias/Comissioning10-v4/RAW",
                                 primary="MinimumBias", processed="Comissioning10-v4", tier="RAW",
                                 dbsurl="https://cmsweb-prod.cern.ch/dbs/prod/global/DBSReader",
                                 block_blacklist=[], block_whitelist=[],
                                 run_blacklist=[], run_whitelist=[])
        procTask.data.constraints.sites.whitelist = []
        procTask.data.constraints.sites.blacklist = []

        procTaskCmsswHelper = procTaskCmssw.getTypeHelper()
        procTaskCmsswHelper.setGlobalTag("TestGlobalTag::All")
        procTaskCmsswHelper.cmsswSetup("CMSSW_3_5_8_patch3", softwareEnvironment = "",
                                       scramArch = "slc5_amd64_gcc434")

        procTaskCmsswHelper.setDataProcessingConfig("cosmics", "PromptReco")

        if self.emulation:
            procTaskStageOutHelper = procTaskStageOut.getTypeHelper()
            procTaskLogArchHelper  = procTaskLogArch.getTypeHelper()
            procTaskCmsswHelper.data.emulator.emulatorName = "CMSSW"
            procTaskStageOutHelper.data.emulator.emulatorName = "StageOut"
            procTaskLogArchHelper.data.emulator.emulatorName = "LogArchive"

        return procTask

    def addLogCollectTask(self, parentTask, taskName = "LogCollect"):
        """
        _addLogCollecTask_

        Create a LogCollect task for log archives that are produced by the
        parent task.
        """
        logCollectTask = parentTask.addTask(taskName)
        logCollectStep = logCollectTask.makeStep("logCollect1")
        logCollectStep.setStepType("LogCollect")
        logCollectTask.applyTemplates()
        logCollectTask.setSplittingAlgorithm("MinFileBased", files_per_job = 500)
        logCollectTask.setTaskType("LogCollect")

        parentTaskLogArch = parentTask.getStep("logArch1")
        logCollectTask.setInputReference(parentTaskLogArch, outputModule = "logArchive")
        return

    def addOutputModule(self, parentTask, outputModuleName, dataTier, filterName):
        """
        _addOutputModule_

        Add an output module to the geven processing task.  This will also
        create merge and cleanup tasks for the output of the output module.
        A handle to the merge task is returned to make it easy to use the merged
        output of the output module as input to another task.
        """
        cmsswStep = parentTask.getStep("cmsRun1")
        cmsswStepHelper = cmsswStep.getTypeHelper()
        cmsswStepHelper.addOutputModule(outputModuleName,
                                        primaryDataset = "MinimumBias",
                                        processedDataset = "Commissioning10-v4",
                                        dataTier = "RAW",
                                        lfnBase = "/store/temp/WMAgent/unmerged",
                                        mergedLFNBase = "/store/temp/WMAgent/merged")
        cmsswStepHelper.addOutputModule(outputModuleName,
                                        primaryDataset = "MinimumBias",
                                        processedDataset = "Commissioning10-v4",
                                        dataTier = "RECO",
                                        lfnBase = "/store/temp/WMAgent/unmerged",
                                        mergedLFNBase = "/store/temp/WMAgent/merged")
        cmsswStepHelper.addOutputModule(outputModuleName,
                                        primaryDataset = "MinimumBias",
                                        processedDataset = "Commissioning10-v4",
                                        dataTier = "DQM",
                                        lfnBase = "/store/temp/WMAgent/unmerged",
                                        mergedLFNBase = "/store/temp/WMAgent/merged")
        return

    def __call__(self, emulation = False):
        """
        _call_

        Create a test workload.
        """
        self.emulation = emulation

        workload = self.createWorkload()
        procTask = workload.newTask("ReReco")

        self.setupProcessingTask(procTask)
        self.addLogCollectTask(procTask)

        self.addOutputModule(procTask, "TestOutputModule", "RECO", "SomeFilter")
        return workload

def testWorkload(emulation = False):
    """
    _testWorkflow_

    Instantiate the TestWorkloadFactory and create a workload.
    """
    myTestWorkloadFactory = TestWorkloadFactory()
    return myTestWorkloadFactory(emulation = emulation)

if __name__ == "__main__":
    testWorkload()



#!/usr/bin/env python
"""
Unittest for WMTweak module

"""

import unittest

import PSetTweaks.WMTweak as WMTweaks
from PSetTweaks.WMTweak import WMTweakMaskError
from PSetTweaks.PSetTweak import PSetTweak

from WMCore.DataStructs.Mask import Mask
from WMCore.DataStructs.Job import Job

class WMTweakTest(unittest.TestCase):


    def testInit(self):
        """
        _testInit_
        Tests the tweakMaker initialization
        """

        try:
            tweak = WMTweaks.TweakMaker()
        except Exception as ex:
            msg = "Failed to instantiate WMTweak:\n"
            msg += str(ex)
            self.fail(msg)

    def testFirstEvent(self):
        """
        _testFirstEvent_

        Verify that we set the firstEvent parameter whenever the FirstEvent
        field in the job mask is a positive integer. And the job is
        not production.
        """
        job = Job()
        job["input_files"] = [{"lfn": "bogusFile", "parents": []}]
        job["mask"] = Mask()

        tweak = PSetTweak()
        WMTweaks.makeJobTweak(job, tweak)

        self.assertFalse(hasattr(tweak.process.source, "skipEvents"),
                         "Error: There should be no skipEvents tweak.")
        self.assertFalse(hasattr(tweak.process.source, "firstEvent"),
                         "Error: There should be no firstEvent tweak.")

        job["mask"]["FirstEvent"] = 0
        tweak = PSetTweak()
        WMTweaks.makeJobTweak(job, tweak)

        self.assertTrue(hasattr(tweak.process.source, "skipEvents"),
                        "Error: There should be a skipEvents tweak.")
        self.assertEqual(tweak.process.source.skipEvents, 'customTypeCms.untracked.uint32(0)',
                         "Error: The skipEvents tweak should be 0.")
        return

    def testFirstRun(self):
        """
        _testFirstRun_
        Verify that when we set the FirstRun in the mask, it is set in the
        process but when it is not, then no firstRun appears in the process.
        This for jobs with real input files.
        """
        job = Job()
        job["input_files"] = [{"lfn": "bogusFile", "parents": []}]
        job["mask"] = Mask()

        tweak = PSetTweak()
        WMTweaks.makeJobTweak(job, tweak)

        self.assertFalse(hasattr(tweak.process.source, "firstRun"),
                         "Error: There should be no firstRun tweak.")

        job["mask"]["FirstRun"] = 93
        tweak = WMTweaks.makeJobTweak(job, tweak)

        self.assertTrue(hasattr(tweak.process.source, "firstRun"),
                        "Error: There should be a firstRun tweak.")
        self.assertEqual(tweak.process.source.firstRun, 'customTypeCms.untracked.uint32(93)',
                         "Error: The firstRun tweak should be 93.")
        return

    def testFirstEventMC(self):
        """
        _testFirstEventMC_

        Verify that we set the firstEvent parameter whenever the FirstEvent
        field in the job mask is a positive integer and the job is a production
        one. Otherwise we get a number based on the counter (modulo 2^32 - 1)
        """
        job = Job()
        job["input_files"] = [{"lfn": "MCFakeFile", "parents": []}]
        job["mask"] = Mask()
        job["counter"] = 5
        job["mask"]["FirstLumi"] = 200

        try:
            tweak = PSetTweak()
            WMTweaks.makeJobTweak(job, tweak)
            self.assertRaises(WMTweakMaskError,  WMTweaks.makeJobTweak, job)
        except WMTweakMaskError:
            pass

        job["mask"]["FirstEvent"] = 100
        tweak = PSetTweak()
        WMTweaks.makeJobTweak(job, tweak)
        self.assertFalse(hasattr(tweak.process.source, "skipEvents"),
                        "Error: There should be no skipEvents tweak, it's MC.")
        self.assertTrue(hasattr(tweak.process.source, "firstEvent"),
                        "Error: There should be a first event tweak")
        self.assertEqual(tweak.process.source.firstEvent, 'customTypeCms.untracked.uint32(100)',
                         "Error: The firstEvent tweak should be 100.")
        return

    def testFirstLumiMC(self):
        """
        _testFirstLumiMC_
        Verify that we set the lumi in a MC job and it gets into
        process.source.firstRun parameter, and if we don't at least we
        get the counter there.
        """
        job = Job()
        job["input_files"] = [{"lfn": "MCFakeFile", "parents": []}]
        job["mask"] = Mask()
        job["counter"] = 5
        job["mask"]["FirstEvent"] = 100

        try:
            tweak = PSetTweak()
            WMTweaks.makeJobTweak(job, tweak)
            self.assertRaises(WMTweakMaskError,  WMTweaks.makeJobTweak, job)
        except WMTweakMaskError:
            pass

        job["mask"]["FirstLumi"] = 200
        tweak =  PSetTweak()
        WMTweaks.makeJobTweak(job, tweak)

        self.assertTrue(hasattr(tweak.process.source, "firstLuminosityBlock"),
                        "Error: There should be a first lumi tweak")
        self.assertEqual(tweak.process.source.firstLuminosityBlock, 'customTypeCms.untracked.uint32(200)',
                       "Error: The first luminosity block should be 200")

        job["mask"]["FirstLumi"] = 10
        tweak = PSetTweak()
        WMTweaks.makeJobTweak(job, tweak)

        self.assertTrue(hasattr(tweak.process.source, "firstLuminosityBlock"),
                        "Error: There should be a first lumi tweak")
        self.assertEqual(tweak.process.source.firstLuminosityBlock, 'customTypeCms.untracked.uint32(10)',
                       "Error: The first luminosity block should be 10")

    def testFirstRunMC(self):
        """
        _testFirstRunMC_
        Verify that we set the lumi in a MC job and it gets into
        process.source.firstRun parameter.
        """
        job = Job()
        job["input_files"] = [{"lfn": "MCFakeFile", "parents": []}]
        job["mask"] = Mask()
        job["mask"]["FirstLumi"] = 200
        job["mask"]["FirstEvent"] = 100
        job["counter"] = 5

        tweak = PSetTweak()
        WMTweaks.makeJobTweak(job, tweak)

        self.assertTrue(hasattr(tweak.process.source, "firstRun"),
                        "Error: There should be a first run tweak")
        self.assertEqual(tweak.process.source.firstRun, 'customTypeCms.untracked.uint32(1)',
                       "Error: The first run should be 1")

        job["mask"]["FirstRun"] = 5
        tweak = PSetTweak()
        WMTweaks.makeJobTweak(job, tweak)

        self.assertTrue(hasattr(tweak.process.source, "firstRun"),
                        "Error: There should be a first run tweak")
        self.assertEqual(tweak.process.source.firstRun, 'customTypeCms.untracked.uint32(5)',
                       "Error: The first run should be 5")
if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python
"""
_SiblingProcessingBased_t_

Test SiblingProcessing job splitting.
"""

import unittest
import threading

from WMCore.WMBS.File import File
from WMCore.WMBS.Fileset import Fileset
from WMCore.WMBS.Subscription import Subscription
from WMCore.WMBS.Workflow import Workflow

from WMCore.DAOFactory import DAOFactory
from WMCore.JobSplitting.SplitterFactory import SplitterFactory
from WMQuality.TestInit import TestInit

class SiblingProcessingBasedTest(unittest.TestCase):
    """
    _SiblingProcessingBasedTest_

    Test SiblingProcessing job splitting.
    """
    def setUp(self):
        """
        _setUp_

        Setup the database connections and schema.
        """
        self.testInit = TestInit(__file__)
        self.testInit.setLogging()
        self.testInit.setDatabaseConnection()
        self.testInit.setSchema(customModules = ["WMCore.WMBS"],
                                useDefault = False)

        myThread = threading.currentThread()
        daofactory = DAOFactory(package = "WMCore.WMBS",
                                logger = myThread.logger,
                                dbinterface = myThread.dbi)

        locationAction = daofactory(classname = "Locations.New")
        locationAction.execute("T2_CH_CERN", pnn = "T2_CH_CERN")
        locationAction.execute("T1_US_FNAL", pnn = "T1_US_FNAL_Disk")

        self.testFilesetA = Fileset(name = "FilesetA")
        self.testFilesetA.create()
        self.testFilesetB = Fileset(name = "FilesetB")
        self.testFilesetB.create()

        self.testFileA = File("testFileA", size = 1000, events = 100,
                              locations = set(["T2_CH_CERN"]))
        self.testFileA.create()
        self.testFileB = File("testFileB", size = 1000, events = 100,
                              locations = set(["T2_CH_CERN"]))
        self.testFileB.create()
        self.testFileC = File("testFileC", size = 1000, events = 100,
                              locations = set(["T2_CH_CERN"]))
        self.testFileC.create()

        self.testFilesetA.addFile(self.testFileA)
        self.testFilesetA.addFile(self.testFileB)
        self.testFilesetA.addFile(self.testFileC)
        self.testFilesetA.commit()

        self.testFileD = File("testFileD", size = 1000, events = 100,
                              locations = set(["T2_CH_CERN"]))
        self.testFileD.create()
        self.testFileE = File("testFileE", size = 1000, events = 100,
                              locations = set(["T2_CH_CERN"]))
        self.testFileE.create()
        self.testFileF = File("testFileF", size = 1000, events = 100,
                              locations = set(["T2_CH_CERN"]))
        self.testFileF.create()

        self.testFilesetB.addFile(self.testFileD)
        self.testFilesetB.addFile(self.testFileE)
        self.testFilesetB.addFile(self.testFileF)
        self.testFilesetB.commit()

        testWorkflowA = Workflow(spec = "specA.xml", owner = "Steve",
                                 name = "wfA", task = "Test")
        testWorkflowA.create()
        testWorkflowB = Workflow(spec = "specB.xml", owner = "Steve",
                                 name = "wfB", task = "Test")
        testWorkflowB.create()
        testWorkflowC = Workflow(spec = "specC.xml", owner = "Steve",
                                 name = "wfC", task = "Test")
        testWorkflowC.create()
        testWorkflowD = Workflow(spec = "specD.xml", owner = "Steve",
                                 name = "wfD", task = "Test")
        testWorkflowD.create()

        self.testSubscriptionA = Subscription(fileset = self.testFilesetA,
                                              workflow = testWorkflowA,
                                              split_algo = "FileBased",
                                              type = "Processing")
        self.testSubscriptionA.create()
        self.testSubscriptionB = Subscription(fileset = self.testFilesetB,
                                              workflow = testWorkflowB,
                                              split_algo = "FileBased",
                                              type = "Processing")
        self.testSubscriptionB.create()
        self.testSubscriptionC = Subscription(fileset = self.testFilesetB,
                                              workflow = testWorkflowC,
                                              split_algo = "FileBased",
                                              type = "Processing")
        self.testSubscriptionC.create()
        self.testSubscriptionD = Subscription(fileset = self.testFilesetB,
                                              workflow = testWorkflowD,
                                              split_algo = "FileBased",
                                              type = "Processing")
        self.testSubscriptionD.create()

        deleteWorkflow = Workflow(spec = "specE.xml", owner = "Steve",
                                  name = "wfE", task = "Test")
        deleteWorkflow.create()

        self.deleteSubscriptionA = Subscription(fileset = self.testFilesetA,
                                                workflow = deleteWorkflow,
                                                split_algo = "SiblingProcessingBased",
                                                type = "Cleanup")
        self.deleteSubscriptionA.create()
        self.deleteSubscriptionB = Subscription(fileset = self.testFilesetB,
                                                workflow = deleteWorkflow,
                                                split_algo = "SiblingProcessingBased",
                                                type = "Cleanup")
        self.deleteSubscriptionB.create()
        return

    def tearDown(self):
        """
        _tearDown_

        Clear out WMBS.
        """
        self.testInit.clearDatabase()
        return

    def testSiblingProcessing(self):
        """
        _testSiblingProcessing_

        Verify that the sibling processing split works correctly dealing with
        failed files and acquiring files correctly.
        """
        splitter = SplitterFactory()
        deleteFactoryA = splitter(package = "WMCore.WMBS",
                                  subscription = self.deleteSubscriptionA)
        deleteFactoryB = splitter(package = "WMCore.WMBS",
                                  subscription = self.deleteSubscriptionB)

        result = deleteFactoryA()

        assert len(result) == 0, \
               "Error: No jobs should be returned."

        result = deleteFactoryB()

        assert len(result) == 0, \
               "Error: No jobs should be returned."

        self.testSubscriptionA.completeFiles(self.testFileA)

        result = deleteFactoryA(files_per_job = 1)

        assert len(result) == 1, \
               "Error: Only one jobgroup should be returned."
        assert len(result[0].jobs) == 1, \
               "Error: There should only be one job in the jobgroup."
        assert result[0].jobs[0]["possiblePSN"] == set(["T2_CH_CERN"]), \
               "Error: possiblePSN is wrong."
        assert len(result[0].jobs[0]["input_files"]) == 1, \
               "Error: Job should only have one input file."
        assert result[0].jobs[0]["input_files"][0]["lfn"] == "testFileA", \
               "Error: Input file for job is wrong."

        result = deleteFactoryB(files_per_job = 1)

        assert len(result) == 0, \
               "Error: Second subscription should have no jobs."

        result = deleteFactoryA(files_per_job = 1)

        assert len(result) == 0, \
               "Error: No jobs should have been created."

        self.testSubscriptionB.completeFiles(self.testFileD)
        self.testSubscriptionC.failFiles(self.testFileD)

        result = deleteFactoryA(files_per_job = 1)

        assert len(result) == 0, \
               "Error: No jobs should have been created."

        result = deleteFactoryB(files_per_job = 1)

        assert len(result) == 0, \
               "Error: No jobs should have been created."

        self.testSubscriptionD.failFiles(self.testFileD)

        result = deleteFactoryA(files_per_job = 1)

        assert len(result) == 0, \
               "Error: No jobs should have been created."

        result = deleteFactoryB(files_per_job = 1)

        assert len(result) == 0, \
               "Error: No job groups should have been created."

        self.testSubscriptionB.completeFiles([self.testFileE, self.testFileF])
        self.testSubscriptionC.completeFiles([self.testFileE, self.testFileF])
        self.testSubscriptionD.completeFiles([self.testFileE, self.testFileF])

        result = deleteFactoryB(files_per_job = 10)

        assert len(result) == 0, \
               "Error: No jobs should have been created."

        self.testFilesetB.markOpen(False)

        result = deleteFactoryB(files_per_job = 10)

        assert len(result) == 1, \
               "Error: One jobgroup should have been returned."
        assert len(result[0].jobs) == 1, \
               "Error: There should only be one job in the jobgroup."
        assert len(result[0].jobs[0]["input_files"]) == 2, \
               "Error: Job should only have one input file."

        lfns = [result[0].jobs[0]["input_files"][0]["lfn"], result[0].jobs[0]["input_files"][1]["lfn"]]

        assert "testFileE" in lfns, \
               "Error: TestFileE missing from job input."
        assert "testFileF" in lfns, \
               "Error: TestFileF missing from job input."

        self.assertEqual(len(self.deleteSubscriptionB.availableFiles()), 0,
                         "Error: There should be no available files.")

        completeFiles = self.deleteSubscriptionB.filesOfStatus("Completed")
        self.assertEqual(len(completeFiles), 1,
                         "Error: There should only be one complete file.")
        self.assertEqual(list(completeFiles)[0]["lfn"], "testFileD",
                         "Error: Test file D should be complete.")

        return

    def testMultipleLocations(self):
        """
        _testMultipleLocations_

        Verify that the sibling processing based algorithm doesn't create jobs
        that run over files at multiple sites.
        """
        testFile1 = File("testFile1", size = 1000, events = 100,
                         locations = set(["T1_US_FNAL_Disk"]))
        testFile1.create()
        testFile2 = File("testFile2", size = 1000, events = 100,
                         locations = set(["T1_US_FNAL_Disk"]))
        testFile2.create()
        testFile3 = File("testFile3", size = 1000, events = 100,
                         locations = set(["T1_US_FNAL_Disk"]))
        testFile3.create()

        self.testFilesetA.addFile(testFile1)
        self.testFilesetA.addFile(testFile2)
        self.testFilesetA.addFile(testFile3)
        self.testFilesetA.commit()
        self.testFilesetA.markOpen(False)

        self.testSubscriptionA.completeFiles([testFile1, testFile2, testFile3])
        self.testSubscriptionA.completeFiles([self.testFileA, self.testFileB, self.testFileC])

        splitter = SplitterFactory()
        deleteFactoryA = splitter(package = "WMCore.WMBS",
                                  subscription = self.deleteSubscriptionA)

        result = deleteFactoryA(files_per_job = 50)

        assert len(result) == 2, \
               "Error: Wrong number of jobgroups returned."

        goldenFilesA = ["testFileA", "testFileB", "testFileC"]
        goldenFilesB = ["testFile1", "testFile2", "testFile3"]

        for jobGroup in result:
            assert len(jobGroup.jobs) == 1, \
                   "Error: Wrong number of jobs in jobgroup."
            assert len(jobGroup.jobs[0]["input_files"]) == 3, \
                   "Error: Wrong number of input files in job."

            jobSite = jobGroup.jobs[0]["possiblePSN"]

            assert (jobSite == set(["T2_CH_CERN"])
                    or jobSite == set(["T1_US_FNAL"])), \
                    "Error: Wrong site for job."

            if jobSite == set(["T2_CH_CERN"]):
                goldenFiles = goldenFilesA
            else:
                goldenFiles = goldenFilesB

            for jobFile in jobGroup.jobs[0]["input_files"]:
                goldenFiles.remove(jobFile["lfn"])

            assert len(goldenFiles) == 0,  \
                   "Error: Files are missing."

        return

    def testLargeNumberOfFiles(self):
        """
        _testLargeNumberOfFiles_

        Setup a subscription with 500 files and verify that the splitting algo
        works correctly.
        """
        testWorkflowA = Workflow(spec = "specA.xml", owner = "Steve",
                                 name = "wfA", task = "Test")
        testWorkflowA.create()
        testWorkflowB = Workflow(spec = "specB.xml", owner = "Steve",
                                 name = "wfB", task = "Test")
        testWorkflowB.create()

        testFileset = Fileset(name = "TestFileset")
        testFileset.create()

        allFiles = []
        for i in range(500):
            testFile = File(str(i), size = 1000, events = 100,
                            locations = set(["T2_CH_CERN"]))
            testFile.create()
            allFiles.append(testFile)
            testFileset.addFile(testFile)
        testFileset.commit()

        testSubscriptionA = Subscription(fileset = testFileset,
                                         workflow = testWorkflowA,
                                         split_algo = "FileBased",
                                         type = "Processing")
        testSubscriptionA.create()
        testSubscriptionB = Subscription(fileset = testFileset,
                                         workflow = testWorkflowB,
                                         split_algo = "SiblingProcessingBased",
                                         type = "Processing")
        testSubscriptionB.create()

        testSubscriptionA.completeFiles(allFiles)

        splitter = SplitterFactory()
        deleteFactoryA = splitter(package = "WMCore.WMBS",
                                  subscription = testSubscriptionB)

        result = deleteFactoryA(files_per_job = 50)
        self.assertEqual(len(result), 1,
                         "Error: Wrong number of job groups returned.")
        self.assertEqual(len(result[0].jobs), 10,
                         "Error: Wrong number of jobs returned.")

        return

    def testFilesWithoutOtherSubscriptions(self):
        """
        _testFilesWithoutOtherSubscriptions_

        Test the case where files only in the delete subscription
        can happen if cleanup of the other subscriptions is fast

        """
        testWorkflowA = Workflow(spec = "specA.xml", owner = "Steve",
                                 name = "wfA", task = "Test")
        testWorkflowA.create()

        testFileset = Fileset(name = "TestFileset")
        testFileset.create()

        allFiles = []
        for i in range(500):
            testFile = File(str(i), size = 1000, events = 100,
                            locations = set(["T2_CH_CERN"]))
            testFile.create()
            allFiles.append(testFile)
            testFileset.addFile(testFile)
        testFileset.commit()

        testSubscriptionA = Subscription(fileset = testFileset,
                                         workflow = testWorkflowA,
                                         split_algo = "SiblingProcessingBased",
                                         type = "Processing")
        testSubscriptionA.create()

        splitter = SplitterFactory()
        deleteFactoryA = splitter(package = "WMCore.WMBS",
                                  subscription = testSubscriptionA)

        result = deleteFactoryA(files_per_job = 50)
        self.assertEqual(len(result), 1,
                         "Error: Wrong number of job groups returned.")
        self.assertEqual(len(result[0].jobs), 10,
                         "Error: Wrong number of jobs returned.")

        return

if __name__ == '__main__':
    unittest.main()



# python-future
from builtins import range

# system modules
import json
import re
import zlib
from multiprocessing import Process

import cherrypy
from cherrypy import response
from cheroot.test import webtest

# WMCore modules
from WMCore.REST.Server import RESTApi, RESTEntity, restcall, rows
from WMCore.REST.Test import setup_dummy_server, fake_authz_headers
from WMCore.REST.Test import fake_authz_key_file
from WMCore.REST.Validation import validate_num, validate_str
from WMCore.REST.Error import InvalidObject
from WMCore.REST.Format import RawFormat
from WMCore.REST.Tools import tools

gif_bytes = (b'GIF89a\x01\x00\x01\x00\x82\x00\x01\x99"\x1e\x00\x00\x00\x00\x00'
             b'\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00\x00'
             b'\x00,\x00\x00\x00\x00\x01\x00\x01\x00\x02\x03\x02\x08\t\x00;')

FAKE_FILE = fake_authz_key_file()
PORT = 8887

class Simple(RESTEntity):
    def validate(self, *args): pass

    @restcall
    @tools.expires(secs=300)
    def get(self):
        return rows(['foo'])

class Multi(RESTEntity):
    def validate(self, apiobj, method, api, param, safe):
        validate_str("etag", param, safe, re.compile("^[a-z]*$"), optional=True)
        validate_num("lim", param, safe, bare=True, optional=True, minval=0, maxval=10)

    def _generate(self, lim):
        for i in range(0, 10):
            if i == lim:
                raise InvalidObject("cut at %d" % i)
            yield ["row", i]

        if i == lim:
            raise InvalidObject("cut at %d" % i)

    @restcall
    @tools.expires(secs=300)
    def get(self, lim, etag):
        if etag:
            response.headers["ETag"] = '"%s"' % etag

        if lim == 0:
            raise InvalidObject("cut at 0")

        return self._generate(lim)

class Image(RESTEntity):
    def validate(self, *args): pass

    @restcall(formats=[("image/gif", RawFormat())])
    @tools.expires(secs=300)
    def get(self):
        return gif_bytes

class Root(RESTApi):
    def __init__(self, app, config, mount):
        RESTApi.__init__(self, app, config, mount)
        self._add({ "simple": Simple(app, self, config, mount),
                    "image":  Image(app, self, config, mount),
                    "multi":  Multi(app, self, config, mount) })

class Tester(webtest.WebCase):

    def setUp(self):
        self.h = fake_authz_headers(FAKE_FILE.data)
        webtest.WebCase.PORT = PORT
        self.engine = cherrypy.engine
        self.proc = load_server(self.engine)

    def tearDown(self):
        stop_server(self.proc, self.engine)

    def _test_accept_ok(self, fmt, page = "/test/simple", inbody = None):
        h = self.h + [("Accept", fmt)]
        self.getPage(page, headers = h)
        self.assertStatus("200 OK")
        if fmt.find("*") >= 0:
            self.assertHeader("Content-Type")
        else:
            self.assertHeader("Content-Type", fmt)
        self.assertHeader("X-REST-Status", "100")
        self.assertHeader("X-REST-Time")
        self.assertNoHeader("X-Error-ID")
        self.assertNoHeader("X-Error-HTTP")
        self.assertNoHeader("X-Error-Info")
        self.assertNoHeader("X-Error-Detail")
        if inbody:
            self.assertInBody(inbody)

    def _test_accept_fail(self, fmt, page="/test/simple",
                          avail="application/json, application/xml"):
        h = self.h + [("Accept", fmt)]
        self.getPage(page, headers = h)
        self.assertStatus("406 Not Acceptable")
        self.assertHeader("X-REST-Status", "201")
        self.assertHeader("X-REST-Time")
        self.assertHeader("X-Error-ID")
        self.assertHeader("X-Error-HTTP", "406")
        self.assertHeader("X-Error-Info", "Available types: %s" % avail)
        self.assertHeader("X-Error-Detail", "Not acceptable")
        self.assertInBody("Not acceptable")

    def test_accept_good_json(self):
        self._test_accept_ok("application/json", inbody="foo")

    def test_accept_good_xml(self):
        self._test_accept_ok("application/xml", inbody="foo")

    def test_accept_good_gif(self):
        self._test_accept_ok("image/gif", page="/test/image")
        self.assertBody(gif_bytes)

    def test_accept_good_gif2(self):
        self._test_accept_ok("image/*", page="/test/image")
        self.assertBody(gif_bytes)

    def test_accept_reject_octets(self):
        self._test_accept_fail("application/octet-stream")

    def test_accept_reject_xfoo(self):
        self._test_accept_fail("application/x-foo")

    def test_accept_reject_gif(self):
        self._test_accept_fail("image/gif")

    def test_accept_reject_gif2(self):
        self._test_accept_fail("image/png", page="/test/image", avail="image/gif")

    def test_accept_reject_png(self):
        self._test_accept_fail("image/png")

    def test_simple_json(self):
        h = self.h
        h.append(("Accept", "application/json"))
        self.getPage("/test/simple", headers = h)
        self.assertStatus("200 OK")
        b = json.loads(self.body)
        assert isinstance(b, dict)
        assert "desc" not in b
        assert "result" in b
        assert isinstance(b["result"], list)
        assert len(b["result"]) == 1
        assert b["result"][0] == "foo"

    def test_simple_json_deflate(self):
        h = self.h
        h.append(("Accept", "application/json"))
        h.append(("Accept-Encoding", "deflate"))
        self.getPage("/test/simple", headers = h)
        self.assertStatus("200 OK")
        self.assertHeader("Content-Length")
        self.assertHeader("Content-Encoding", "deflate")
        b = json.loads(zlib.decompress(self.body, -zlib.MAX_WBITS))
        assert isinstance(b, dict)
        assert "desc" not in b
        assert "result" in b
        assert isinstance(b["result"], list)
        assert len(b["result"]) == 1
        assert b["result"][0] == "foo"

    def test_multi_nothrow(self):
        h = self.h
        h.append(("Accept", "application/json"))
        self.getPage("/test/multi", headers = h)
        self.assertStatus("200 OK")
        self.assertHeader("X-REST-Status", "100")
        b = json.loads(self.body)
        assert isinstance(b, dict)
        assert "desc" not in b
        assert "result" in b
        assert isinstance(b["result"], list)
        assert len(b["result"]) == 10
        for i in range(0, 10):
            assert isinstance(b["result"][i], list)
            assert b["result"][i][0] == "row"
            assert b["result"][i][1] == i

    def test_multi_throw0(self):
        h = self.h
        h.append(("Accept", "application/json"))
        self.getPage("/test/multi?lim=0", headers = h)
        self.assertStatus(400)
        self.assertHeader("X-REST-Status", "306")
        self.assertHeader("X-Error-HTTP", "400")
        self.assertHeader("X-Error-Info", "cut at 0")
        self.assertHeader("X-Error-Detail", "Invalid object")
        self.assertHeader("X-Error-ID")

    def test_multi_throw5a(self):
        h = self.h
        h.append(("Accept", "application/json"))
        self.getPage("/test/multi?lim=5&etag=x", headers = h)
        self.assertStatus("200 OK")
        self.assertHeader("X-REST-Status", "100")
        b = json.loads(self.body)
        assert isinstance(b, dict)
        assert "desc" not in b
        assert "result" in b
        assert isinstance(b["result"], list)
        assert len(b["result"]) == 5
        for i in range(0, 5):
            assert isinstance(b["result"][i], list)
            assert b["result"][i][0] == "row"
            assert b["result"][i][1] == i

    def test_multi_throw5b(self):
        h = self.h
        h.append(("Accept", "application/json"))
        self.getPage("/test/multi?lim=5", headers = h)
        self.assertStatus(400)
        self.assertHeader("X-REST-Status", "200")
        self.assertHeader("X-Error-HTTP", "400")
        self.assertHeader("X-Error-Info", "cut at 5")
        self.assertHeader("X-Error-Detail", "Invalid object")
        self.assertHeader("X-Error-ID")

    def test_multi_throw10(self):
        h = self.h
        h.append(("Accept", "application/json"))
        self.getPage("/test/multi?lim=10&etag=x", headers = h)
        self.assertStatus("200 OK")
        self.assertHeader("X-REST-Status", "100")
        b = json.loads(self.body)
        assert isinstance(b, dict)
        assert "desc" not in b
        assert "result" in b
        assert isinstance(b["result"], list)
        assert len(b["result"]) == 10
        for i in range(0, 10):
            assert isinstance(b["result"][i], list)
            assert b["result"][i][0] == "row"
            assert b["result"][i][1] == i

def setup_server():
    srcfile = __file__.split("/")[-1].split(".py")[0]
    setup_dummy_server(srcfile, "Root", authz_key_file=FAKE_FILE, port=PORT)

def load_server(engine):
    setup_server()
    proc = Process(target=start_server, name="cherrypy_Api_t", args=(engine,))
    proc.start()
    proc.join(timeout=1)
    return proc

def start_server(engine):
    webtest.WebCase.PORT = PORT
    cherrypy.log.screen = True
    engine.start()
    engine.block()

def stop_server(proc, engine):
    cherrypy.log.screen = True
    engine.stop()
    proc.terminate()

if __name__ == '__main__':
    webtest.main()



#!/usr/bin/env python
# -*- coding: ISO-8859-1 -*-


from __future__ import division, print_function

import unittest

from WMCore.Wrappers.JsonWrapper.JSONThunker import JSONThunker


class testJSONThunker(unittest.TestCase):
    """
    Direct tests of thunking standard python type
    """

    def setUp(self):
        self.thunker = JSONThunker()

    def roundTrip(self, data):
        encoded = self.thunker.thunk(data)
        decoded = self.thunker.unthunk(encoded)
        self.assertEqual(data, decoded)

    def testStr(self):
        self.roundTrip('hello')

    def testList(self):
        self.roundTrip([123, 456])

    def testDict(self):
        self.roundTrip({'abc': 123, 'def': 456})
        self.roundTrip({'abc': 123, 456: 'def'})

    def testSet(self):
        self.roundTrip(set([]))
        self.roundTrip(set([123, 'abc']))


if __name__ == "__main__":
    unittest.main()



#from Integration_t.ReRecoLifeCycle_t import ReRecoLifeCycle_t
from Integration_t.RequestLifeCycleBase_t import RequestLifeCycleBase_t

import time
import unittest
import nose
from nose.plugins.attrib import attr


class RequestCancellation_t(RequestLifeCycleBase_t, unittest.TestCase):

    requestParams = {'CMSSWVersion' : 'CMSSW_4_4_2_patch2',
                        "Scenario": "pp", "inputMode" : "Scenario", "ProcScenario" : "pp",
                        "RequestType": "ReReco",
                        "Requestor": 'integration',
                        "InputDataset": "/BTag/Run2011B-v1/RAW", "RunWhitelist" : [177316, 177317],
                        "GlobalTag": 'FT_R_44_V11::All', 'Group' : 'DMWM', "RequestPriority" : 10,
                    }

    # instead of checking its running cancel it.
    @attr("lifecycle")
    def test60CancelRequest(self):
        """Cancel workflow"""
        self.__class__.reqmgr.reportRequestStatus(self.__class__.request_name, 'aborted')
        self.__class__.request = self.__class__.reqmgr.getRequest(self.__class__.request_name)
        self.assertEqual(self.__class__.request['RequestStatus'], 'aborted')

    @attr("lifecycle")
    def test70WorkQueueFinished(self):
        """Request canceled in workqueue"""
        if not self.__class__.request_name:
            raise nose.SkipTest
        start = time.time()
        while True:
            request = [x for x in self.__class__.workqueue.getElementsCountAndJobsByWorkflow() if \
                    x == self.__class__.request_name]
            # request deleted from wq shortly after finishing, so may not appear here
            if not request or request == [x for x in request if x['status'] in ('Canceled')]:
                break
            if start + (60 * 20) < time.time():
                raise RuntimeError('timeout waiting for request to finish')
            time.sleep(15)

    @attr("lifecycle")
    def test80RequestFinished(self):
        """Request canceled"""
        if not self.__class__.request_name:
            raise nose.SkipTest
        start = time.time()
        while True:
            self.__class__.request = self.__class__.reqmgr.getRequest(self.__class__.request_name)
            if self.__class__.request['RequestStatus'] == 'aborted':
                break
            if start + (60 * 20) < time.time():
                raise RuntimeError('timeout waiting for request to finish')
            time.sleep(15)

# Remove tests that don't apply
delattr(RequestLifeCycleBase_t, 'test60RequestRunning')
delattr(RequestLifeCycleBase_t, 'test90RequestCloseOut')

if __name__ == "__main__":
    unittest.main()



"""
_InsertComponent_

MySQL implementation of Block.New
"""

__all__ = []

from WMCore.Database.DBFormatter import DBFormatter


class InsertComponent(DBFormatter):
    deleteWorkerSql = """DELETE FROM wm_workers
                            WHERE component_id = (SELECT id FROM wm_components
                                   WHERE name = :name)
                       """

    deleteSql = """DELETE FROM wm_components
                       WHERE name = :name"""

    sql = """INSERT INTO wm_components (name, pid, update_threshold)
             VALUES (:name, :pid, :update_threshold)"""

    def execute(self, name, pid, update_threshold,
                conn=None, transaction=False):
        deleteBinds = {"name": name}

        self.dbi.processData(self.deleteWorkerSql, deleteBinds, conn=conn,
                             transaction=transaction)
        self.dbi.processData(self.deleteSql, deleteBinds, conn=conn,
                             transaction=transaction)
        binds = {"name": name, "pid": pid, "update_threshold": update_threshold}
        self.dbi.processData(self.sql, binds, conn=conn,
                             transaction=transaction)
        return



#!/usr/bin/env python
"""
_WMBSMergeBySize_t

Unit tests for generic WMBS merging.
"""

import threading
import unittest

from WMCore.DAOFactory import DAOFactory
from WMCore.DataStructs.Run import Run
from WMCore.JobSplitting.SplitterFactory import SplitterFactory
from WMCore.WMBS.File import File
from WMCore.WMBS.Fileset import Fileset
from WMCore.WMBS.Job import Job
from WMCore.WMBS.JobGroup import JobGroup
from WMCore.WMBS.Subscription import Subscription
from WMCore.WMBS.Workflow import Workflow
from WMQuality.TestInit import TestInit


class WMBSMergeBySize(unittest.TestCase):
    def setUp(self):
        """
        _setUp_

        Boiler plate DB setup.
        """
        self.testInit = TestInit(__file__)
        self.testInit.setLogging()
        self.testInit.setDatabaseConnection()
        self.testInit.setSchema(customModules=["WMCore.WMBS"],
                                useDefault=False)

        myThread = threading.currentThread()
        self.daoFactory = DAOFactory(package="WMCore.WMBS",
                                     logger=myThread.logger,
                                     dbinterface=myThread.dbi)
        return

    def tearDown(self):
        """
        _tearDown_

        Clear out WMBS.
        """
        self.testInit.clearDatabase()
        return

    def stuffWMBS(self, injected=True):
        """
        _stuffWMBS_

        Insert some dummy jobs, jobgroups, filesets, files and subscriptions
        into WMBS to test job creation.  Three completed job groups each
        containing several files are injected.  Another incomplete job group is
        also injected.  Also files are added to the "Mergeable" subscription as
        well as to the output fileset for their jobgroups.
        """
        locationAction = self.daoFactory(classname="Locations.New")
        locationAction.execute(siteName="T2_CH_CERN", pnn="T2_CH_CERN")
        locationAction.execute(siteName="T1_US_FNAL", pnn="T2_CH_CERN")

        changeStateDAO = self.daoFactory(classname="Jobs.ChangeState")

        self.mergeFileset = Fileset(name="mergeFileset")
        self.mergeFileset.create()
        self.bogusFileset = Fileset(name="bogusFileset")
        self.bogusFileset.create()

        self.mergeMergedFileset = Fileset(name="mergeMergedFileset")
        self.mergeMergedFileset.create()
        self.bogusMergedFileset = Fileset(name="bogusMergedFileset")
        self.bogusMergedFileset.create()

        mergeWorkflow = Workflow(name="mergeWorkflow", spec="bunk2",
                                 owner="Steve", task="Test")
        mergeWorkflow.create()
        markWorkflow = self.daoFactory(classname="Workflow.MarkInjectedWorkflows")
        markWorkflow.execute(names=[mergeWorkflow.name], injected=injected)

        self.mergeSubscription = Subscription(fileset=self.mergeFileset,
                                              workflow=mergeWorkflow,
                                              split_algo="WMBSMergeBySize")
        self.mergeSubscription.create()
        self.bogusSubscription = Subscription(fileset=self.bogusFileset,
                                              workflow=mergeWorkflow,
                                              split_algo="WMBSMergeBySize")

        inputFileset = Fileset(name="inputFileset")
        inputFileset.create()

        inputWorkflow = Workflow(name="inputWorkflow", spec="input",
                                 owner="Steve", task="Test")
        inputWorkflow.create()
        inputWorkflow.addOutput("output", self.mergeFileset,
                                self.mergeMergedFileset)
        inputWorkflow.addOutput("output2", self.bogusFileset,
                                self.bogusMergedFileset)
        bogusInputWorkflow = Workflow(name="bogusInputWorkflow", spec="input",
                                      owner="Steve", task="Test")
        bogusInputWorkflow.create()

        inputSubscription = Subscription(fileset=inputFileset,
                                         workflow=inputWorkflow)
        inputSubscription.create()
        bogusInputSubscription = Subscription(fileset=inputFileset,
                                              workflow=bogusInputWorkflow)
        bogusInputSubscription.create()

        parentFile1 = File(lfn="parentFile1")
        parentFile1.create()
        parentFile2 = File(lfn="parentFile2")
        parentFile2.create()
        parentFile3 = File(lfn="parentFile3")
        parentFile3.create()
        parentFile4 = File(lfn="parentFile4")
        parentFile4.create()
        self.parentFileSite2 = File(lfn="parentFileSite2")
        self.parentFileSite2.create()

        jobGroup1 = JobGroup(subscription=inputSubscription)
        jobGroup1.create()
        jobGroup2 = JobGroup(subscription=inputSubscription)
        jobGroup2.create()
        jobGroup3 = JobGroup(subscription=bogusInputSubscription)
        jobGroup3.create()

        testJob1 = Job()
        testJob1.addFile(parentFile1)
        testJob1.create(jobGroup1)
        testJob1["state"] = "cleanout"
        testJob1["oldstate"] = "new"
        testJob1["couch_record"] = "somejive"
        testJob1["retry_count"] = 0
        testJob1["outcome"] = "success"
        testJob1.save()
        changeStateDAO.execute([testJob1])

        testJob1A = Job()
        testJob1A.addFile(parentFile1)
        testJob1A.create(jobGroup3)
        testJob1A["state"] = "cleanout"
        testJob1A["oldstate"] = "new"
        testJob1A["couch_record"] = "somejive"
        testJob1A["retry_count"] = 0
        testJob1A["outcome"] = "failure"
        testJob1A.save()
        changeStateDAO.execute([testJob1A])

        testJob2 = Job()
        testJob2.addFile(parentFile2)
        testJob2.create(jobGroup1)
        testJob2["state"] = "cleanout"
        testJob2["oldstate"] = "new"
        testJob2["couch_record"] = "somejive"
        testJob2["retry_count"] = 0
        testJob2["outcome"] = "success"
        testJob2.save()
        changeStateDAO.execute([testJob2])

        testJob3 = Job()
        testJob3.addFile(parentFile3)
        testJob3.create(jobGroup2)
        testJob3["state"] = "cleanout"
        testJob3["oldstate"] = "new"
        testJob3["couch_record"] = "somejive"
        testJob3["retry_count"] = 0
        testJob3["outcome"] = "success"
        testJob3.save()
        changeStateDAO.execute([testJob3])

        testJob4 = Job()
        testJob4.addFile(parentFile4)
        testJob4.create(jobGroup2)
        testJob4["state"] = "cleanout"
        testJob4["oldstate"] = "new"
        testJob4["couch_record"] = "somejive"
        testJob4["retry_count"] = 0
        testJob4["outcome"] = "failure"
        testJob4.save()
        changeStateDAO.execute([testJob4])

        # We'll simulate a failed split by event job that the merger should
        # ignore.
        parentFile5 = File(lfn="parentFile5")
        parentFile5.create()

        testJob5 = Job()
        testJob5.addFile(parentFile5)
        testJob5.create(jobGroup2)
        testJob5["state"] = "cleanout"
        testJob5["oldstate"] = "new"
        testJob5["couch_record"] = "somejive"
        testJob5["retry_count"] = 0
        testJob5["outcome"] = "success"
        testJob5.save()
        changeStateDAO.execute([testJob5])

        testJob6 = Job()
        testJob6.addFile(parentFile5)
        testJob6.create(jobGroup2)
        testJob6["state"] = "cleanout"
        testJob6["oldstate"] = "new"
        testJob6["couch_record"] = "somejive"
        testJob6["retry_count"] = 0
        testJob6["outcome"] = "failure"
        testJob6.save()
        changeStateDAO.execute([testJob6])

        testJob7 = Job()
        testJob7.addFile(self.parentFileSite2)
        testJob7.create(jobGroup2)
        testJob7["state"] = "cleanout"
        testJob7["oldstate"] = "new"
        testJob7["couch_record"] = "somejive"
        testJob7["retry_count"] = 0
        testJob7["outcome"] = "success"
        testJob7.save()
        changeStateDAO.execute([testJob7])

        badFile1 = File(lfn="badFile1", size=10241024, events=10241024,
                        first_event=0, locations={"T2_CH_CERN"})
        badFile1.addRun(Run(1, *[45]))
        badFile1.create()
        badFile1.addParent(parentFile5["lfn"])

        file1 = File(lfn="file1", size=1024, events=1024, first_event=0,
                     locations={"T2_CH_CERN"})
        file1.addRun(Run(1, *[45]))
        file1.create()
        file1.addParent(parentFile1["lfn"])
        file2 = File(lfn="file2", size=1024, events=1024,
                     first_event=1024, locations={"T2_CH_CERN"})
        file2.addRun(Run(1, *[45]))
        file2.create()
        file2.addParent(parentFile1["lfn"])
        file3 = File(lfn="file3", size=1024, events=1024,
                     first_event=2048, locations={"T2_CH_CERN"})
        file3.addRun(Run(1, *[45]))
        file3.create()
        file3.addParent(parentFile1["lfn"])
        file4 = File(lfn="file4", size=1024, events=1024,
                     first_event=3072, locations={"T2_CH_CERN"})
        file4.addRun(Run(1, *[45]))
        file4.create()
        file4.addParent(parentFile1["lfn"])

        fileA = File(lfn="fileA", size=1024, events=1024,
                     first_event=0, locations={"T2_CH_CERN"})
        fileA.addRun(Run(1, *[46]))
        fileA.create()
        fileA.addParent(parentFile2["lfn"])
        fileB = File(lfn="fileB", size=1024, events=1024,
                     first_event=1024, locations={"T2_CH_CERN"})
        fileB.addRun(Run(1, *[46]))
        fileB.create()
        fileB.addParent(parentFile2["lfn"])
        fileC = File(lfn="fileC", size=1024, events=1024,
                     first_event=2048, locations={"T2_CH_CERN"})
        fileC.addRun(Run(1, *[46]))
        fileC.create()
        fileC.addParent(parentFile2["lfn"])

        fileI = File(lfn="fileI", size=1024, events=1024,
                     first_event=0, locations={"T2_CH_CERN"})
        fileI.addRun(Run(2, *[46]))
        fileI.create()
        fileI.addParent(parentFile3["lfn"])
        fileII = File(lfn="fileII", size=1024, events=1024,
                      first_event=1024, locations={"T2_CH_CERN"})
        fileII.addRun(Run(2, *[46]))
        fileII.create()
        fileII.addParent(parentFile3["lfn"])
        fileIII = File(lfn="fileIII", size=1024, events=1024,
                       first_event=2048, locations={"T2_CH_CERN"})
        fileIII.addRun(Run(2, *[46]))
        fileIII.create()
        fileIII.addParent(parentFile3["lfn"])
        fileIV = File(lfn="fileIV", size=1024, events=1024,
                      first_event=3072, locations={"T2_CH_CERN"})
        fileIV.addRun(Run(2, *[46]))
        fileIV.create()
        fileIV.addParent(parentFile3["lfn"])

        fileX = File(lfn="badFileA", size=1024, events=1024,
                     first_event=0, locations={"T2_CH_CERN"})
        fileX.addRun(Run(1, *[47]))
        fileX.create()
        fileX.addParent(parentFile4["lfn"])
        fileY = File(lfn="badFileB", size=1024, events=1024,
                     first_event=1024, locations={"T2_CH_CERN"})
        fileY.addRun(Run(1, *[47]))
        fileY.create()
        fileY.addParent(parentFile4["lfn"])
        fileZ = File(lfn="badFileC", size=1024, events=1024,
                     first_event=2048, locations={"T2_CH_CERN"})
        fileZ.addRun(Run(1, *[47]))
        fileZ.create()
        fileZ.addParent(parentFile4["lfn"])

        jobGroup1.output.addFile(file1)
        jobGroup1.output.addFile(file2)
        jobGroup1.output.addFile(file3)
        jobGroup1.output.addFile(file4)
        jobGroup1.output.addFile(fileA)
        jobGroup1.output.addFile(fileB)
        jobGroup1.output.addFile(fileC)
        jobGroup1.output.commit()

        jobGroup2.output.addFile(fileI)
        jobGroup2.output.addFile(fileII)
        jobGroup2.output.addFile(fileIII)
        jobGroup2.output.addFile(fileIV)
        jobGroup2.output.addFile(fileX)
        jobGroup2.output.addFile(fileY)
        jobGroup2.output.addFile(fileZ)
        jobGroup2.output.addFile(badFile1)
        jobGroup2.output.commit()

        for fileObj in [file1, file2, file3, file4, fileA, fileB, fileC, fileI,
                        fileII, fileIII, fileIV, fileX, fileY, fileZ, badFile1]:
            self.mergeFileset.addFile(fileObj)
            self.bogusFileset.addFile(fileObj)

        self.mergeFileset.commit()
        self.bogusFileset.commit()

        return

    def testMinMergeSize1(self):
        """
        _testMinMergeSize1_

        Set the minimum merge size to be 20,000 bytes which is more than the
        sum of all file sizes in the WMBS instance.  Verify that no merge jobs
        will be produced.
        """
        self.stuffWMBS()

        splitter = SplitterFactory()
        jobFactory = splitter(package="WMCore.WMBS",
                              subscription=self.mergeSubscription)

        result = jobFactory(min_merge_size=20000, max_merge_size=2000000000,
                            max_merge_events=200000000)

        assert len(result) == 0, \
            "ERROR: No job groups should be returned."

        return

    def testMinMergeSize1a(self):
        """
        _testMinMergeSize1a_

        Set the minimum merge size to be 20,000 bytes which is more than the
        sum of all file sizes in the WMBS instance and mark the fileset as
        closed.  Verify that one job containing all files is pushed out.
        """
        self.stuffWMBS()
        self.mergeFileset.markOpen(False)

        splitter = SplitterFactory()
        jobFactory = splitter(package="WMCore.WMBS",
                              subscription=self.mergeSubscription)

        result = jobFactory(min_merge_size=20000, max_merge_size=200000,
                            max_merge_events=20000)

        assert len(result) == 1, \
            "ERROR: More than one JobGroup returned: %s" % len(result)

        assert len(result[0].jobs) == 2, \
            "Error: Two jobs should have been returned."

        goldenFilesA = ["file1", "file2", "file3", "file4", "fileA", "fileB",
                        "fileC"]
        goldenFilesB = ["fileI", "fileII", "fileIII", "fileIV"]

        for job in result[0].jobs:

            self.assertEqual(job["possiblePSN"], {"T1_US_FNAL", "T2_CH_CERN"})

            jobFiles = job.getFiles()

            if len(jobFiles) == len(goldenFilesA):
                self.assertEqual(job["estimatedDiskUsage"], 7)
                goldenFiles = goldenFilesA
            else:
                self.assertEqual(job["estimatedDiskUsage"], 4)
                goldenFiles = goldenFilesB

            currentRun = 0
            currentLumi = 0
            currentEvent = 0
            for fileObj in jobFiles:
                fileObj.loadData()
                assert fileObj["lfn"] in goldenFiles, \
                    "Error: Unknown file: %s" % fileObj["lfn"]
                goldenFiles.remove(fileObj["lfn"])

                fileRun = list(fileObj["runs"])[0].run
                fileLumi = min(list(fileObj["runs"])[0])
                fileEvent = fileObj["first_event"]

                if currentRun == 0:
                    currentRun = fileRun
                    currentLumi = fileLumi
                    currentEvent = fileEvent
                    continue

                assert fileRun >= currentRun, \
                    "ERROR: Files not sorted by run."

                if fileRun == currentRun:
                    assert fileLumi >= currentLumi, \
                        "ERROR: Files not ordered by lumi"

                if fileLumi == currentLumi:
                    assert fileEvent >= currentEvent, \
                        "ERROR: Files not ordered by first event"

                currentRun = fileRun
                currentLumi = fileLumi
                currentEvent = fileEvent

        return

    def testMinMergeSize2(self):
        """
        _testMinMergeSize2_

        Set the minimum merge size to be 7,167 bytes which is one byte less
        than the sum of all the file sizes in the largest merge group in the
        WMBS instance.  Verify that one merge job containing all the files in
        the largest merge group is produced.
        """
        self.stuffWMBS()

        splitter = SplitterFactory()
        jobFactory = splitter(package="WMCore.WMBS",
                              subscription=self.mergeSubscription)

        result = jobFactory(min_merge_size=7167, max_merge_size=20000,
                            max_merge_events=20000)

        assert len(result) == 1, \
            "ERROR: More than one JobGroup returned: %d" % len(result)

        assert len(result[0].jobs) == 1, \
            "ERROR: One job should have been returned."

        self.assertEqual(result[0].jobs[0]["estimatedDiskUsage"], 7)

        self.assertEqual(result[0].jobs[0]["possiblePSN"], {"T1_US_FNAL", "T2_CH_CERN"})

        jobFiles = list(result[0].jobs)[0].getFiles()

        goldenFiles = ["file1", "file2", "file3", "file4", "fileA", "fileB",
                       "fileC"]

        assert len(jobFiles) == len(goldenFiles), \
            "ERROR: Merge job should contain %d files." % len(goldenFiles)

        currentRun = 0
        currentLumi = 0
        currentEvent = 0
        for fileObj in jobFiles:
            assert fileObj["lfn"] in goldenFiles, \
                "Error: Unknown file: %s" % fileObj["lfn"]
            goldenFiles.remove(fileObj["lfn"])

            fileRun = list(fileObj["runs"])[0].run
            fileLumi = min(list(fileObj["runs"])[0])
            fileEvent = fileObj["first_event"]

            if currentRun == 0:
                currentRun = fileRun
                currentLumi = fileLumi
                currentEvent = fileEvent
                continue

            assert fileRun >= currentRun, \
                "ERROR: Files not sorted by run."

            if fileRun == currentRun:
                assert fileLumi >= currentLumi, \
                    "ERROR: Files not ordered by lumi"

                if fileLumi == currentLumi:
                    assert fileEvent >= currentEvent, \
                        "ERROR: Files not ordered by first event"

            currentRun = fileRun
            currentLumi = fileLumi
            currentEvent = fileEvent

        return

    def testMaxMergeSize1(self):
        """
        _testMaxMergeSize1_

        Set the maximum merge size to be two bytes.  Verify that three merge
        jobs are created, one for each job group that exists inside the WMBS
        instance.  Verify that each merge job contains the expected files.
        """
        self.stuffWMBS()

        splitter = SplitterFactory()
        jobFactory = splitter(package="WMCore.WMBS",
                              subscription=self.mergeSubscription)

        result = jobFactory(min_merge_size=1, max_merge_size=2,
                            max_merge_events=20000)

        assert len(result) == 1, \
            "ERROR: More than one JobGroup returned: %s" % result

        assert len(result[0].jobs) == 3, \
            "ERROR: Three jobs should have been returned."

        self.assertEqual(result[0].jobs[0]["possiblePSN"], {"T1_US_FNAL", "T2_CH_CERN"})

        goldenFilesA = ["file1", "file2", "file3", "file4"]
        goldenFilesB = ["fileA", "fileB", "fileC"]
        goldenFilesC = ["fileI", "fileII", "fileIII", "fileIV"]

        for job in result[0].jobs:
            jobFiles = job.getFiles()

            if jobFiles[0]["lfn"] in goldenFilesA:
                self.assertEqual(job["estimatedDiskUsage"], 4)
                goldenFiles = goldenFilesA
            elif jobFiles[0]["lfn"] in goldenFilesB:
                self.assertEqual(job["estimatedDiskUsage"], 3)
                goldenFiles = goldenFilesB
            else:
                self.assertEqual(job["estimatedDiskUsage"], 4)
                goldenFiles = goldenFilesC

            currentRun = 0
            currentLumi = 0
            currentEvent = 0
            for fileObj in jobFiles:
                assert fileObj["lfn"] in goldenFiles, \
                    "Error: Unknown file in merge jobs."
                goldenFiles.remove(fileObj["lfn"])

                fileRun = list(fileObj["runs"])[0].run
                fileLumi = min(list(fileObj["runs"])[0])
                fileEvent = fileObj["first_event"]

                if currentRun == 0:
                    continue

                assert fileRun >= currentRun, \
                    "ERROR: Files not sorted by run."

                if fileRun == currentRun:
                    assert fileLumi >= currentLumi, \
                        "ERROR: Files not ordered by lumi"

                    if fileLumi == currentLumi:
                        assert fileEvent >= currentEvent, \
                            "ERROR: Files not ordered by first event"

        assert len(goldenFilesA) == 0 and len(goldenFilesB) == 0 and \
               len(goldenFilesC) == 0, \
            "ERROR: Files missing from merge jobs."

        return

    def testMaxMergeSize2(self):
        """
        _testMaxMergeSize2_

        Set the minimum merge size to be one byte larger than the largest job
        group in the WMBS instance and the max merge size to be one byte larger
        than the total size of two of the groups.  Verify that one merge job
        is produced with two of the job groups in it.
        """
        self.stuffWMBS()

        splitter = SplitterFactory()
        jobFactory = splitter(package="WMCore.WMBS",
                              subscription=self.mergeSubscription)

        result = jobFactory(min_merge_size=4097, max_merge_size=7169,
                            max_merge_events=20000)

        assert len(result) == 1, \
            "ERROR: More than one JobGroup returned."

        assert len(result[0].jobs) == 1, \
            "ERROR: One job should have been returned."

        goldenFilesA = ["file1", "file2", "file3", "file4"]
        goldenFilesB = ["fileA", "fileB", "fileC"]
        goldenFilesC = ["fileI", "fileII", "fileIII", "fileIV"]

        self.assertEqual(result[0].jobs[0]["estimatedDiskUsage"], 7)

        self.assertEqual(result[0].jobs[0]["possiblePSN"], {"T1_US_FNAL", "T2_CH_CERN"})

        jobFiles = list(result[0].jobs)[0].getFiles()

        currentRun = 0
        currentLumi = 0
        currentEvent = 0
        for fileObj in jobFiles:

            if fileObj["lfn"] in goldenFilesA:
                goldenFilesA.remove(fileObj["lfn"])
            elif fileObj["lfn"] in goldenFilesB:
                goldenFilesB.remove(fileObj["lfn"])
            elif fileObj["lfn"] in goldenFilesC:
                goldenFilesC.remove(fileObj["lfn"])

            fileRun = list(fileObj["runs"])[0].run
            fileLumi = min(list(fileObj["runs"])[0])
            fileEvent = fileObj["first_event"]

            if currentRun == 0:
                currentRun = fileRun
                currentLumi = fileLumi
                currentEvent = fileEvent
                continue

            assert fileRun >= currentRun, \
                "ERROR: Files not sorted by run."

            if fileRun == currentRun:
                assert fileLumi >= currentLumi, \
                    "ERROR: Files not ordered by lumi"

                if fileLumi == currentLumi:
                    assert fileEvent >= currentEvent, \
                        "ERROR: Files not ordered by first event"

            currentRun = fileRun
            currentLumi = fileLumi
            currentEvent = fileEvent

        assert len(goldenFilesB) == 0 and \
               (len(goldenFilesA) == 0 or len(goldenFilesC) == 0), \
            "ERROR: Files not allocated to jobs correctly."

        return

    def testMaxEvents1(self):
        """
        _testMaxEvents1_

        Set the maximum number of events per merge job to 1.
        """
        self.stuffWMBS()

        splitter = SplitterFactory()
        jobFactory = splitter(package="WMCore.WMBS",
                              subscription=self.mergeSubscription)

        result = jobFactory(min_merge_size=1, max_merge_size=20000,
                            max_merge_events=1)

        assert len(result) == 1, \
            "ERROR: More than one JobGroup returned: %s" % result

        assert len(result[0].jobs) == 3, \
            "ERROR: Three jobs should have been returned: %s" % len(result[0].jobs)

        goldenFilesA = ["file1", "file2", "file3", "file4"]
        goldenFilesB = ["fileA", "fileB", "fileC"]
        goldenFilesC = ["fileI", "fileII", "fileIII", "fileIV"]

        for job in result[0].jobs:

            self.assertEqual(job["possiblePSN"], {"T1_US_FNAL", "T2_CH_CERN"})

            jobFiles = job.getFiles()

            if jobFiles[0]["lfn"] in goldenFilesA:
                self.assertEqual(job["estimatedDiskUsage"], 4)
                goldenFiles = goldenFilesA
            elif jobFiles[0]["lfn"] in goldenFilesB:
                self.assertEqual(job["estimatedDiskUsage"], 3)
                goldenFiles = goldenFilesB
            else:
                self.assertEqual(job["estimatedDiskUsage"], 4)
                goldenFiles = goldenFilesC

            currentRun = 0
            currentLumi = 0
            currentEvent = 0
            for fileObj in jobFiles:
                assert fileObj["lfn"] in goldenFiles, \
                    "Error: Unknown file in merge jobs."
                goldenFiles.remove(fileObj["lfn"])

                fileRun = list(fileObj["runs"])[0].run
                fileLumi = min(list(fileObj["runs"])[0])
                fileEvent = fileObj["first_event"]

                if currentRun == 0:
                    currentRun = fileRun
                    currentLumi = fileLumi
                    currentEvent = fileEvent
                    continue

                assert fileRun >= currentRun, \
                    "ERROR: Files not sorted by run: %s, %s" % (fileRun, currentRun)

                if fileRun == currentRun:
                    assert fileLumi >= currentLumi, \
                        "ERROR: Files not ordered by lumi"

                    if fileLumi == currentLumi:
                        assert fileEvent >= currentEvent, \
                            "ERROR: Files not ordered by first event"

                currentRun = fileRun
                currentLumi = fileLumi
                currentEvent = fileEvent

        assert len(goldenFilesA) == 0 and len(goldenFilesB) == 0 and \
               len(goldenFilesC) == 0, \
            "ERROR: Files missing from merge jobs."

        return

    def testMaxEvents2(self):
        """
        _testMaxEvents2_

        Set the minimum merge size to be one byte larger than the largest job
        group in the WMBS instance and the max events to be one event larger
        than the total events in two of the groups.  Verify that one merge job
        is produced with two of the job groups in it.
        """
        self.stuffWMBS()

        splitter = SplitterFactory()
        jobFactory = splitter(package="WMCore.WMBS",
                              subscription=self.mergeSubscription)

        result = jobFactory(min_merge_size=4097, max_merge_size=20000,
                            max_merge_events=7169)

        assert len(result) == 1, \
            "ERROR: More than one JobGroup returned."

        assert len(result[0].jobs) == 1, \
            "ERROR: One job should have been returned."

        self.assertEqual(result[0].jobs[0]["estimatedDiskUsage"], 7)

        self.assertEqual(result[0].jobs[0]["possiblePSN"], {"T1_US_FNAL", "T2_CH_CERN"})

        goldenFilesA = ["file1", "file2", "file3", "file4"]
        goldenFilesB = ["fileA", "fileB", "fileC"]
        goldenFilesC = ["fileI", "fileII", "fileIII", "fileIV"]

        jobFiles = list(result[0].jobs)[0].getFiles()

        currentRun = 0
        currentLumi = 0
        currentEvent = 0
        for fileObj in jobFiles:

            if fileObj["lfn"] in goldenFilesA:
                goldenFilesA.remove(fileObj["lfn"])
            elif fileObj["lfn"] in goldenFilesB:
                goldenFilesB.remove(fileObj["lfn"])
            elif fileObj["lfn"] in goldenFilesC:
                goldenFilesC.remove(fileObj["lfn"])

            fileRun = list(fileObj["runs"])[0].run
            fileLumi = min(list(fileObj["runs"])[0])
            fileEvent = fileObj["first_event"]

            if currentRun == 0:
                currentRun = fileRun
                currentLumi = fileLumi
                currentEvent = fileEvent
                continue

            assert fileRun >= currentRun, \
                "ERROR: Files not sorted by run."

            if fileRun == currentRun:
                assert fileLumi >= currentLumi, \
                    "ERROR: Files not ordered by lumi"

                if fileLumi == currentLumi:
                    assert fileEvent >= currentEvent, \
                        "ERROR: Files not ordered by first event"

            currentRun = fileRun
            currentLumi = fileLumi
            currentEvent = fileEvent

        assert len(goldenFilesB) == 0 and \
               (len(goldenFilesA) == 0 or len(goldenFilesC) == 0), \
            "ERROR: Files not allocated to jobs correctly."

        return

    def testParallelProcessing(self):
        """
        _testParallelProcessing_

        Verify that merging works correctly when multiple processing
        subscriptions are run over the same input files.  The merging algorithm
        should ignore processing jobs that feed into different merge
        subscriptions.
        """
        locationAction = self.daoFactory(classname="Locations.New")
        locationAction.execute(siteName="T2_CH_CERN", pnn="T2_CH_CERN")
        locationAction.execute(siteName="T1_US_FNAL", pnn="T2_CH_CERN")

        mergeFilesetA = Fileset(name="mergeFilesetA")
        mergeFilesetB = Fileset(name="mergeFilesetB")
        mergeFilesetA.create()
        mergeFilesetB.create()

        mergeMergedFilesetA = Fileset(name="mergeMergedFilesetA")
        mergeMergedFilesetB = Fileset(name="mergeMergedFilesetB")
        mergeMergedFilesetA.create()
        mergeMergedFilesetB.create()

        mergeWorkflow = Workflow(name="mergeWorkflow", spec="bogus",
                                 owner="Steve", task="Test")
        mergeWorkflow.create()

        mergeSubscriptionA = Subscription(fileset=mergeFilesetA,
                                          workflow=mergeWorkflow,
                                          split_algo="WMBSMergeBySize")
        mergeSubscriptionB = Subscription(fileset=mergeFilesetB,
                                          workflow=mergeWorkflow,
                                          split_algo="WMBSMergeBySize")
        mergeSubscriptionA.create()
        mergeSubscriptionB.create()

        inputFileset = Fileset(name="inputFileset")
        inputFileset.create()

        inputFileA = File(lfn="inputLFNA")
        inputFileB = File(lfn="inputLFNB")
        inputFileA.create()
        inputFileB.create()

        procWorkflowA = Workflow(name="procWorkflowA", spec="bunk2",
                                 owner="Steve", task="Test")
        procWorkflowA.create()
        procWorkflowA.addOutput("output", mergeFilesetA, mergeMergedFilesetA)
        procWorkflowB = Workflow(name="procWorkflowB", spec="bunk3",
                                 owner="Steve", task="Test2")
        procWorkflowB.create()
        procWorkflowB.addOutput("output", mergeFilesetB, mergeMergedFilesetB)

        procSubscriptionA = Subscription(fileset=inputFileset,
                                         workflow=procWorkflowA,
                                         split_algo="EventBased")
        procSubscriptionA.create()
        procSubscriptionB = Subscription(fileset=inputFileset,
                                         workflow=procWorkflowB,
                                         split_algo="EventBased")
        procSubscriptionB.create()

        jobGroupA = JobGroup(subscription=procSubscriptionA)
        jobGroupA.create()
        jobGroupB = JobGroup(subscription=procSubscriptionB)
        jobGroupB.create()

        changeStateDAO = self.daoFactory(classname="Jobs.ChangeState")

        testJobA = Job()
        testJobA.addFile(inputFileA)
        testJobA.create(jobGroupA)
        testJobA["state"] = "cleanout"
        testJobA["oldstate"] = "new"
        testJobA["couch_record"] = "somejive"
        testJobA["retry_count"] = 0
        testJobA["outcome"] = "success"
        testJobA.save()

        testJobB = Job()
        testJobB.addFile(inputFileB)
        testJobB.create(jobGroupA)
        testJobB["state"] = "cleanout"
        testJobB["oldstate"] = "new"
        testJobB["couch_record"] = "somejive"
        testJobB["retry_count"] = 0
        testJobB["outcome"] = "success"
        testJobB.save()

        testJobC = Job()
        testJobC.addFile(inputFileA)
        testJobC.create(jobGroupB)
        testJobC["state"] = "cleanout"
        testJobC["oldstate"] = "new"
        testJobC["couch_record"] = "somejive"
        testJobC["retry_count"] = 0
        testJobC["outcome"] = "success"
        testJobC.save()

        testJobD = Job()
        testJobD.addFile(inputFileA)
        testJobD.create(jobGroupB)
        testJobD["state"] = "cleanout"
        testJobD["oldstate"] = "new"
        testJobD["couch_record"] = "somejive"
        testJobD["retry_count"] = 0
        testJobD["outcome"] = "failure"
        testJobD.save()

        testJobE = Job()
        testJobE.addFile(inputFileB)
        testJobE.create(jobGroupB)
        testJobE["state"] = "cleanout"
        testJobE["oldstate"] = "new"
        testJobE["couch_record"] = "somejive"
        testJobE["retry_count"] = 0
        testJobE["outcome"] = "success"
        testJobE.save()

        testJobF = Job()
        testJobF.addFile(inputFileB)
        testJobF.create(jobGroupB)
        testJobF["state"] = "cleanout"
        testJobF["oldstate"] = "new"
        testJobF["couch_record"] = "somejive"
        testJobF["retry_count"] = 0
        testJobF["outcome"] = "failure"
        testJobF.save()

        changeStateDAO.execute([testJobA, testJobB, testJobC, testJobD,
                                testJobE, testJobF])

        fileA = File(lfn="fileA", size=1024, events=1024, first_event=0,
                     locations={"T2_CH_CERN"})
        fileA.addRun(Run(1, *[45]))
        fileA.create()
        fileA.addParent(inputFileA["lfn"])
        fileB = File(lfn="fileB", size=1024, events=1024, first_event=0,
                     locations={"T2_CH_CERN"})
        fileB.addRun(Run(1, *[45]))
        fileB.create()
        fileB.addParent(inputFileB["lfn"])

        jobGroupA.output.addFile(fileA)
        jobGroupA.output.addFile(fileB)
        jobGroupA.output.commit()

        mergeFilesetA.addFile(fileA)
        mergeFilesetA.addFile(fileB)
        mergeFilesetA.commit()

        fileC = File(lfn="fileC", size=1024, events=1024, first_event=0,
                     locations={"T2_CH_CERN"})
        fileC.addRun(Run(1, *[45]))
        fileC.create()
        fileC.addParent(inputFileA["lfn"])
        fileD = File(lfn="fileD", size=1024, events=1024, first_event=0,
                     locations={"T2_CH_CERN"})
        fileD.addRun(Run(1, *[45]))
        fileD.create()
        fileD.addParent(inputFileB["lfn"])

        jobGroupB.output.addFile(fileC)
        jobGroupB.output.addFile(fileD)

        mergeFilesetB.addFile(fileC)
        mergeFilesetB.addFile(fileD)
        mergeFilesetB.commit()

        splitter = SplitterFactory()
        jobFactory = splitter(package="WMCore.WMBS",
                              subscription=mergeSubscriptionB)

        result = jobFactory(min_merge_size=1, max_merge_size=20000,
                            max_merge_events=7169)

        assert len(result) == 0, \
            "Error: No merge jobs should have been created."

        fileE = File(lfn="fileE", size=1024, events=1024, first_event=0,
                     locations={"T2_CH_CERN"})
        fileE.addRun(Run(1, *[45]))
        fileE.create()
        fileE.addParent(inputFileA["lfn"])
        fileF = File(lfn="fileF", size=1024, events=1024, first_event=0,
                     locations={"T2_CH_CERN"})
        fileF.addRun(Run(1, *[45]))
        fileF.create()
        fileF.addParent(inputFileB["lfn"])

        jobGroupB.output.addFile(fileE)
        jobGroupB.output.addFile(fileF)

        mergeFilesetB.addFile(fileE)
        mergeFilesetB.addFile(fileF)
        mergeFilesetB.commit()

        testJobD["outcome"] = "success"
        testJobD.save()
        testJobF["outcome"] = "success"
        testJobF.save()

        changeStateDAO.execute([testJobD, testJobF])

        result = jobFactory(min_merge_size=1, max_merge_size=20000,
                            max_merge_events=7169)

        assert len(result) == 1, \
            "Error: One merge job should have been created: %s" % len(result)

        return

    def testLocationMerging(self):
        """
        _testLocationMerging_

        Verify that files residing on different SEs are not merged together in
        the same job.
        """
        self.stuffWMBS()

        locationAction = self.daoFactory(classname="Locations.New")
        locationAction.execute(siteName="T1_UK_RAL", pnn="T1_UK_RAL_Disk")

        fileSite2 = File(lfn="fileSite2", size=4098, events=1024,
                         first_event=0, locations={"T1_UK_RAL_Disk"})
        fileSite2.addRun(Run(1, *[46]))
        fileSite2.create()
        fileSite2.addParent(self.parentFileSite2["lfn"])

        self.mergeFileset.addFile(fileSite2)
        self.mergeFileset.commit()

        splitter = SplitterFactory()
        jobFactory = splitter(package="WMCore.WMBS",
                              subscription=self.mergeSubscription)

        result = jobFactory(min_merge_size=4097, max_merge_size=99999999,
                            max_merge_events=999999999)

        assert len(result) == 1, \
            "ERROR: More than one JobGroup returned."

        assert len(result[0].jobs) == 2, \
            "ERROR: Two jobs should have been returned."

        ralJobs = 0
        fnalcernJobs = 0
        for job in result[0].jobs:
            if job["possiblePSN"] == {"T1_UK_RAL"}:
                ralJobs += 1
            elif job["possiblePSN"] == {"T1_US_FNAL", "T2_CH_CERN"}:
                fnalcernJobs += 1

        self.assertEqual(ralJobs, 1)
        self.assertEqual(fnalcernJobs, 1)

        return

    def testFilesetCloseout(self):
        """
        _testFilesetCloseout_

        Verify that the merge algorithm works correctly when it's input fileset
        is closed.  The split algorithm should create merge jobs for all files
        regardless of size and then mark any orphaned files (files that are the
        result of a split by lumi / split by event where one of the parent
        processing jobs has failed while others have succeeded) as failed so
        that the fileset closing works.
        """
        self.stuffWMBS()

        splitter = SplitterFactory()
        jobFactory = splitter(package="WMCore.WMBS",
                              subscription=self.mergeSubscription)

        # Get out all the good merge jobs out of the way.
        result = jobFactory(min_merge_size=1, max_merge_size=999999999999,
                            max_merge_events=999999999)

        # Verify that the bad files are the only "available" files
        availableAction = self.daoFactory(classname="Subscriptions.GetAvailableFilesMeta")
        availFiles = availableAction.execute(self.mergeSubscription["id"])

        assert len(availFiles) == 4, \
            "Error: Wrong number of available files."

        goldenFiles = ["badFile1", "badFileA", "badFileB", "badFileC"]
        for availFile in availFiles:
            assert availFile["lfn"] in goldenFiles, \
                "Error: Extra file is available."

            goldenFiles.remove(availFile["lfn"])

        self.mergeFileset.markOpen(False)
        result = jobFactory(min_merge_size=1, max_merge_size=999999999999,
                            max_merge_events=999999999)

        assert len(result) == 0, \
            "Error: Merging should have returned zero jobs."

        self.mergeFileset.markOpen(False)

        availFiles2 = availableAction.execute(self.mergeSubscription["id"])

        assert len(availFiles2) == 0, \
            "Error: There should be no more available files."

        failedAction = self.daoFactory(classname="Subscriptions.GetFailedFiles")
        failedFiles = failedAction.execute(self.mergeSubscription["id"])

        assert len(failedFiles) == 4, \
            "Error: Wrong number of failed files: %s" % failedFiles

        goldenIDs = []
        for availFile in availFiles:
            goldenIDs.append(availFile["id"])

        for failedFile in failedFiles:
            assert failedFile["file"] in goldenIDs, \
                "Error: Extra failed file."

        return

    def testFilesetCloseout2(self):
        """
        _testFilesetCloseout2_

        Verify that the fail orphan file code does not fail files that have
        failed for other workflows.
        """
        self.stuffWMBS()
        self.mergeFileset.markOpen(False)

        splitter = SplitterFactory()
        jobFactory = splitter(package="WMCore.WMBS",
                              subscription=self.mergeSubscription)

        # Get out all the good merge jobs out of the way.
        result = jobFactory(min_merge_size=1, max_merge_size=999999999999,
                            max_merge_events=999999999)

        self.assertEqual(len(result), 1, "Error: Wrong number of job groups.")
        self.assertEqual(len(result[0].jobs), 2, "Error: Wrong number of jobs.")

        failedAction = self.daoFactory(classname="Subscriptions.GetFailedFiles")
        failedFiles = failedAction.execute(self.mergeSubscription["id"])

        self.assertEqual(len(failedFiles), 4,
                         "Error: Wrong number of failed files: %s" % failedFiles)
        return

    def testForcedMerge(self):
        """
        _testForcedMerge_

        Repeat testMinMergeSize1a, but with non-injected files to assert that
        this causes no jobgroups to be created.
        """
        self.stuffWMBS(injected=False)
        self.mergeFileset.markOpen(False)

        splitter = SplitterFactory()
        jobFactory = splitter(package="WMCore.WMBS",
                              subscription=self.mergeSubscription)

        result = jobFactory(min_merge_size=20000, max_merge_size=200000,
                            max_merge_events=20000)

        self.assertEqual(len(result), 0)

        return


if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python
"""
_XMLParser_

Read the raw XML output from the cmsRun executable.
"""
from __future__ import division, print_function

import logging
import re

from WMCore.Algorithms.ParseXMLFile import coroutine, xmlFileToNode
from WMCore.DataStructs.Run import Run
from WMCore.FwkJobReport import Report


def reportBuilder(nodeStruct, report, target):
    """
    _reportBuilder_

    Driver for coroutine pipe for building reports from the Node
    structure.
    """
    for node in nodeStruct.children:
        target.send((report, node))


@coroutine
def reportDispatcher(targets):
    """
    _reportDispatcher_

    Top level routine for dispatching the parts of the job report to the
    handlers.
    """
    while True:
        report, node = (yield)
        if node.name != "FrameworkJobReport":
            print("Not Handling: ", node.name)
            # TODO: throw
            continue

        for subnode in node.children:
            if subnode.name == "File":
                targets['File'].send((report, subnode))
            elif subnode.name == "InputFile":
                targets['InputFile'].send((report, subnode))
            elif subnode.name == "AnalysisFile":
                targets['AnalysisFile'].send((report, subnode))
            elif subnode.name == "PerformanceReport":
                targets['PerformanceReport'].send((report, subnode))
            elif subnode.name == "FrameworkError":
                targets['FrameworkError'].send((report, subnode))
            elif subnode.name == "SkippedFile":
                targets['SkippedFile'].send((report, subnode))
            elif subnode.name == "FallbackAttempt":
                targets['FallbackAttempt'].send((report, subnode))
            elif subnode.name == "SkippedEvent":
                targets['SkippedEvent'].send((report, subnode))
            else:
                setattr(report.report.parameters, subnode.name, subnode.text)


@coroutine
def fileHandler(targets):
    """
    _fileHandler_

    coroutine to create files and handle sub data in the appropriate
    dispatchers

    """
    while True:
        report, node = (yield)
        moduleName = None
        moduleNode = [x for x in node.children if x.name == "ModuleLabel"][0]
        moduleName = moduleNode.text

        fileRef = report.addOutputFile(moduleName)
        fileAttrs = {}
        for subnode in node.children:
            if subnode.name == "Inputs":
                targets['Inputs'].send((fileRef, subnode))
            elif subnode.name == "Runs":
                targets['Runs'].send((fileRef, subnode))
            elif subnode.name == "Branches":
                targets['Branches'].send((fileRef, subnode))
            else:
                fileAttrs[subnode.name] = subnode.text

        Report.addAttributesToFile(fileRef, lfn=fileAttrs["LFN"],
                                   pfn=fileAttrs["PFN"], catalog=fileAttrs["Catalog"],
                                   module_label=fileAttrs["ModuleLabel"],
                                   guid=fileAttrs["GUID"],
                                   output_module_class=fileAttrs["OutputModuleClass"],
                                   events=int(fileAttrs["TotalEvents"]),
                                   branch_hash=fileAttrs["BranchHash"])


@coroutine
def inputFileHandler(targets):
    """
    _inputFileHandler_

    coroutine to create input files in the report and dispatch
    sub data down the pipeline

    """
    while True:
        report, node = (yield)
        moduleName = None
        moduleNode = [x for x in node.children if x.name == "ModuleLabel"][0]
        moduleName = moduleNode.text

        fileRef = report.addInputFile(moduleName)
        fileAttrs = {}
        for subnode in node.children:
            if subnode.name == "Runs":
                targets['Runs'].send((fileRef, subnode))
            elif subnode.name == "Branches":
                targets['Branches'].send((fileRef, subnode))
            else:
                fileAttrs[subnode.name] = subnode.text

        Report.addAttributesToFile(fileRef, lfn=fileAttrs["LFN"],
                                   pfn=fileAttrs["PFN"], catalog=fileAttrs["Catalog"],
                                   module_label=fileAttrs["ModuleLabel"],
                                   guid=fileAttrs["GUID"], input_type=fileAttrs["InputType"],
                                   input_source_class=fileAttrs["InputSourceClass"],
                                   events=int(fileAttrs["EventsRead"]))


@coroutine
def analysisFileHandler(targets):
    """
    _analysisFileHandler_

    handle analysis file entries in the report

    """
    while True:
        report, node = (yield)
        filename = None
        attrs = {}
        for subnode in node.children:
            if subnode.name == "FileName":
                filename = subnode.text
            else:
                attrs[subnode.name] = subnode.attrs.get('Value', None)

        report.addAnalysisFile(filename, **attrs)


@coroutine
def errorHandler():
    """
    _errorHandler_

    Handle FrameworkError reports.
    """
    while True:
        report, node = (yield)
        excepcode = node.attrs.get("ExitStatus", 8001)
        exceptype = node.attrs.get("Type", "CMSException")

        # There should be atmost one step in the report at this point in time.
        if len(report.listSteps()) == 0:
            report.addError("unknownStep", excepcode, exceptype, node.text)
        else:
            report.addError(report.listSteps()[0], excepcode, exceptype, node.text)


@coroutine
def skippedFileHandler():
    while True:
        report, node = (yield)
        lfn = node.attrs.get("Lfn", None)
        pfn = node.attrs.get("Pfn", None)
        report.addSkippedFile(lfn, pfn)


@coroutine
def fallbackAttemptHandler():
    while True:
        report, node = (yield)
        lfn = node.attrs.get("Lfn", None)
        pfn = node.attrs.get("Pfn", None)
        report.addFallbackFile(lfn, pfn)


@coroutine
def skippedEventHandler():
    while True:
        report, node = (yield)
        run = node.attrs.get("Run", None)
        event = node.attrs.get("Event", None)
        if run is None:
            continue
        if event is None:
            continue
        report.addSkippedEvent(run, event)


@coroutine
def runHandler():
    """
    _runHandler_

    Sink to add run information to a file.  Given the following XML:
      <Runs>
      <Run ID="122023">
        <LumiSection NEvents="100" ID="215"/>
        <LumiSection NEvents="100" ID="216"/>
      </Run>
      <Run ID="122024">
        <LumiSection ID="1"/>
        <LumiSection ID="2"/>
      </Run>
      </Runs>

    Create a WMCore.DataStructs.Run object for each run and call the
    addRunInfoToFile() function to add the run information to the file
    section.
    """
    while True:
        fileSection, node = (yield)
        for subnode in node.children:
            if subnode.name == "Run":
                runId = subnode.attrs.get("ID", None)
                if runId is None:
                    continue

                lumis = []
                for lumi in subnode.children:
                    if "ID" in lumi.attrs:
                        lumiNumber = int(lumi.attrs['ID'])
                        nEvents = lumi.attrs.get("NEvents", None)
                        if nEvents is not None:
                            try:
                                nEvents = int(nEvents)
                            except ValueError:
                                nEvents = None
                        lumis.append((lumiNumber, nEvents))
                runInfo = Run(runNumber=runId)
                runInfo.extendLumis(lumis)

                Report.addRunInfoToFile(fileSection, runInfo)

@coroutine
def branchHandler():
    """
    _branchHandler_

    Sink to pack branch information into a file.  Given the following XML:
      <Branches>
        <Branch>Branch Name 1</Branch>
        <Branch>Branch Name 2</Branch>
      </Branches>

    Create a list containing all the branch names as use the
    addBranchNamesToFile method to add them to the fileSection.

    Nulled out, we dont need these anyways...

    """
    while True:
        fileSection, node = (yield)
        pass
        # branches = [ subnode.text for subnode in node.children
        #             if subnode.name == "Branch" ]
        # Report.addBranchNamesToFile(fileSection, branches)


@coroutine
def inputAssocHandler():
    """
    _inputAssocHandler_

    Sink to handle output:input association information.  Given the following
    XML:
      <Input>
        <LFN>/path/to/some/lfn.root</LFN>
        <PFN>/some/pfn/info/path/to/some/lfn.root</PFN>
      </Input>

    Extract the LFN and call the addInputToFile() function to associate input to
    output in the FWJR.
    """
    while True:
        fileSection, node = (yield)
        for inputnode in node.children:
            data = {}
            for subnode in inputnode.children:
                data.__setitem__(subnode.name, subnode.text)
            Report.addInputToFile(fileSection, data["LFN"], data['PFN'])


@coroutine
def perfRepHandler(targets):
    """
    _perfRepHandler_

    handle performance report subsections

    """
    while True:
        report, node = (yield)
        perfRep = report.report.performance
        perfRep.section_("summaries")
        perfRep.section_("cpu")
        perfRep.section_("memory")
        perfRep.section_("storage")
        for subnode in node.children:
            metric = subnode.attrs.get('Metric', None)
            if metric == "Timing":
                targets['CPU'].send((perfRep.cpu, subnode))
            elif metric == "SystemMemory" or metric == "ApplicationMemory":
                targets['Memory'].send((perfRep.memory, subnode))
            elif metric == "StorageStatistics":
                targets['Storage'].send((perfRep.storage, subnode))
            else:
                targets['PerformanceSummary'].send((perfRep.summaries,
                                                    subnode))


@coroutine
def perfSummaryHandler():
    """
    _perfSummaryHandler_

    Sink to handle performance summaries

    """
    while True:
        report, node = (yield)
        summary = node.attrs.get('Metric', None)
        if summary is None:
            continue
        # Add performance section if it doesn't exist
        if not hasattr(report, summary):
            report.section_(summary)
        summRep = getattr(report, summary)

        for subnode in node.children:
            setattr(summRep, subnode.attrs['Name'],
                    subnode.attrs['Value'])


@coroutine
def perfCPUHandler():
    """
    _perfCPUHandler_

    sink that packs CPU reports into the job report

    """
    while True:
        report, node = (yield)
        for subnode in node.children:
            setattr(report, subnode.attrs['Name'], subnode.attrs['Value'])


@coroutine
def perfMemHandler():
    """
    _perfMemHandler_

    Pack memory performance reports into the report
    """
    # Make a list of performance info we actually want
    goodStatistics = ['PeakValueRss', 'PeakValueVsize', 'LargestRssEvent-h-PSS']

    while True:
        report, node = (yield)
        for prop in node.children:
            if prop.attrs['Name'] in goodStatistics:
                if prop.attrs['Name'] == 'LargestRssEvent-h-PSS':
                    # need to remove - chars from name as it buggers up downtstream code
                    setattr(report, 'PeakValuePss', prop.attrs['Value'])
                else:
                    setattr(report, prop.attrs['Name'], prop.attrs['Value'])


def checkRegEx(regexp, candidate):
    if re.compile(regexp).match(candidate) == None:
        return False
    return True


@coroutine
def perfStoreHandler():
    """
    _perfStoreHandler_

    Handle the information from the Storage report
    """

    # Make a list of performance info we actually want
    goodStatistics = ['Timing-([a-z]{4})-read(v?)-totalMegabytes',
                      'Timing-([a-z]{4})-write(v?)-totalMegabytes',
                      'Timing-([a-z]{4})-read(v?)-totalMsecs',
                      'Timing-([a-z]{4})-read(v?)-numOperations',
                      'Timing-([a-z]{4})-write(v?)-numOperations',
                      'Timing-([a-z]{4})-read(v?)-maxMsecs',
                      'Timing-tstoragefile-readActual-numOperations',
                      'Timing-tstoragefile-read-numOperations',
                      'Timing-tstoragefile-readViaCache-numSuccessfulOperations',
                      'Timing-tstoragefile-read-numOperations',
                      'Timing-tstoragefile-read-totalMsecs',
                      'Timing-tstoragefile-write-totalMsecs',
                      ]

    while True:
        report, node = (yield)
        logging.debug("Preparing to parse storage statistics")
        storageValues = {}
        for prop in node.children:
            name = prop.attrs['Name']
            for statName in goodStatistics:
                if checkRegEx(statName, name):
                    storageValues[name] = float(prop.attrs['Value'])
                    # setattr(report, name, prop.attrs['Value'])

        writeMethod = None
        readMethod = None
        # Figure out read method
        for key in storageValues:
            if checkRegEx('Timing-([a-z]{4})-read(v?)-numOperations', key):
                if storageValues[key] != 0.0:
                    # This is the reader
                    readMethod = key.split('-')[1]
                    break
        # Figure out the write method
        for key in storageValues:
            if checkRegEx('Timing-([a-z]{4})-write(v?)-numOperations', key):
                if storageValues[key] != 0.0:
                    # This is the reader
                    writeMethod = key.split('-')[1]
                    break

        # Then assemble the information
        # Calculate the values
        logging.debug("ReadMethod: %s", readMethod)
        logging.debug("WriteMethod: %s", writeMethod)
        try:
            readTotalMB = storageValues.get("Timing-%s-read-totalMegabytes" % readMethod, 0) \
                          + storageValues.get("Timing-%s-readv-totalMegabytes" % readMethod, 0)
            readMSecs = storageValues.get("Timing-%s-read-totalMsecs" % readMethod, 0) \
                        + storageValues.get("Timing-%s-readv-totalMsecs" % readMethod, 0)
            totalReads = storageValues.get("Timing-%s-read-numOperations" % readMethod, 0) \
                         + storageValues.get("Timing-%s-readv-numOperations" % readMethod, 0)
            readMaxMSec = max(storageValues.get("Timing-%s-read-maxMsecs" % readMethod, 0),
                              storageValues.get("Timing-%s-readv-maxMsecs" % readMethod, 0))
            readPercOps = storageValues.get("Timing-tstoragefile-readActual-numOperations", 0) / \
                          storageValues.get("Timing-tstoragefile-read-numOperations", 0)
            readCachOps = storageValues.get("Timing-tstoragefile-readViaCache-numSuccessfulOperations", 0) / \
                          storageValues.get("Timing-tstoragefile-read-numOperations", 0)
            readTotalT = storageValues.get("Timing-tstoragefile-read-totalMSecs", 0) / 1000
            readNOps = storageValues.get("Timing-tstoragefile-read-numOperations", 0)
            writeTime = storageValues.get("Timing-tstoragefile-write-totalMsecs", 0) / 1000
            writeTotMB = storageValues.get("Timing-%s-write-totalMegabytes" % writeMethod, 0) \
                         + storageValues.get("Timing-%s-writev-totalMegabytes" % writeMethod, 0)

            if readMSecs > 0:
                readMBSec = readTotalMB / readMSecs * 1000
            else:
                readMBSec = 0
            if totalReads > 0:
                readAveragekB = 1024 * readTotalMB / totalReads
            else:
                readAveragekB = 0

            # Attach them to the report
            setattr(report, 'readTotalMB', readTotalMB)
            setattr(report, 'readMBSec', readMBSec)
            setattr(report, 'readAveragekB', readAveragekB)
            setattr(report, 'readMaxMSec', readMaxMSec)
            setattr(report, 'readPercentageOps', readPercOps)
            setattr(report, 'readTotalSecs', readTotalT)
            setattr(report, 'readNumOps', readNOps)
            setattr(report, 'writeTotalSecs', writeTime)
            setattr(report, 'writeTotalMB', writeTotMB)
            setattr(report, 'readCachePercentageOps', readCachOps)
        except ZeroDivisionError:
            logging.error("Tried to divide by zero doing storage statistics report parsing.")
            logging.error("Either you aren't reading and writing data, or you aren't reporting it.")
            logging.error("Not adding any storage performance info to report.")


def xmlToJobReport(reportInstance, xmlFile):
    """
    _xmlToJobReport_

    parse the XML file and insert the information into the
    Report instance provided

    """
    # read XML, build node structure
    node = xmlFileToNode(xmlFile)

    #  //
    # // Set up coroutine pipeline
    # //
    fileDispatchers = {
        "Runs": runHandler(),
        "Branches": branchHandler(),
        "Inputs": inputAssocHandler(),
    }

    perfRepDispatchers = {
        "PerformanceSummary": perfSummaryHandler(),
        "CPU": perfCPUHandler(),
        "Memory": perfMemHandler(),
        "Storage": perfStoreHandler(),
    }

    dispatchers = {
        "File": fileHandler(fileDispatchers),
        "InputFile": inputFileHandler(fileDispatchers),
        "PerformanceReport": perfRepHandler(perfRepDispatchers),
        "AnalysisFile": analysisFileHandler(fileDispatchers),
        "FrameworkError": errorHandler(),
        "SkippedFile": skippedFileHandler(),
        "FallbackAttempt": fallbackAttemptHandler(),
        "SkippedEvent": skippedEventHandler(),
    }

    #  //
    # // Feed pipeline with node structure and report result instance
    # //
    reportBuilder(
        node, reportInstance,
        reportDispatcher(dispatchers)
    )

    return


childrenMatching = lambda node, nname: [x for x in node.children if x.name == nname]



#!/usr/bin/env python
"""
_ChangeState_

MySQL implementation of Jobs.ChangeState.
Expects a list of job objects with, at least, an id, a state (checked by the
wrapper class), a retry count for that state, and an id for the couchdb record
(also added in by the wrapper class, if not present).
"""

from builtins import map
from WMCore.Database.DBFormatter import DBFormatter

class ChangeState(DBFormatter):
    sql = """UPDATE wmbs_job
            SET state = (select id from wmbs_job_state where name = :state),
                couch_record = :couch_record,
                state_time = :time
            WHERE wmbs_job.id = :job
            """

    def getBinds(self, jobs = []):
        """
        pull out state, couch_record, retry_count and id and apply a timestamp
        via a map function. Return a list of dicts for binds.
        """
        def function(job):
            dict = {'job': job['id'],
                    'state': job['state'],
                    'time': self.timestamp(),
                    'couch_record': job['couch_record']}
            return dict
        return list(map(function, jobs))

    def execute(self, jobs = [], conn = None, transaction = False):
        """
        jobs is a list of Job objects (dicts)
        """
        binds = self.getBinds(jobs)
        self.dbi.processData(self.sql, binds, conn = conn,
                             transaction = transaction)
        return



#!/usr/bin/env python
"""
_ResubmitBlock_t_

WorkQueue ResubmitBlock tests

Created on Feb 19, 2013

@author: dballest
"""

from builtins import range
import os
import unittest
from random import choice

from WMCore.DataStructs.File import File
from WMCore.DataStructs.Run import Run
from WMCore.Database.CMSCouch import CouchServer
from WMCore.GroupUser.User import makeUser
from WMCore.Services.CRIC.CRIC import CRIC
from WMCore.WMSpec.StdSpecs.ReReco import ReRecoWorkloadFactory
from WMCore.WorkQueue.Policy.Start.ResubmitBlock import ResubmitBlock
from WMCore.WorkQueue.WorkQueueExceptions import WorkQueueNoWorkError, WorkQueueWMSpecError
from WMQuality.Emulators.EmulatedUnitTestCase import EmulatedUnitTestCase
from WMQuality.Emulators.WMSpecGenerator.WMSpecGenerator import createConfig
from WMQuality.TestInitCouchApp import TestInitCouchApp


class ResubmitBlockTest(EmulatedUnitTestCase):
    """
    _ResubmitBlockTest
    Tests for WorkQueue splitting for Resubmission workflows

    """

    def __init__(self, methodName='runTest'):
        super(ResubmitBlockTest, self).__init__(methodName=methodName)

    def setUp(self):
        """
        _setUp_

        Setup couchdb and the test environment
        """
        super(ResubmitBlockTest, self).setUp()

        self.group = 'unknown'
        self.user = 'unknown'

        # Set external test helpers
        self.testInit = TestInitCouchApp(__file__)
        self.testInit.setLogging()
        self.testInit.setupCouch("resubmitblock_t", "ACDC", "GroupUser")

        # Define test environment
        self.couchUrl = os.environ["COUCHURL"]
        self.acdcDBName = 'resubmitblock_t'
        self.validLocations = ['T2_US_Nebraska', 'T1_US_FNAL_Disk', 'T1_UK_RAL_Disk']
        self.siteWhitelist = ['T2_XX_SiteA']
        cric = CRIC()
        # Convert phedex node name to a valid processing site name
        self.PSNs = cric.PNNstoPSNs(self.validLocations)
        self.workflowName = 'dballest_ReReco_workflow'
        couchServer = CouchServer(dburl=self.couchUrl)
        self.acdcDB = couchServer.connectDatabase(self.acdcDBName, create=False)
        user = makeUser(self.group, 'sfoulkes@fnal.gov', self.couchUrl, self.acdcDBName)
        user.create()

        return

    def tearDown(self):
        """
        _tearDown_

        Clean couchdb and test environment
        """
        super(ResubmitBlockTest, self).tearDown()

        self.testInit.tearDownCouch()
        return

    def getProcessingACDCSpec(self, splittingAlgo='LumiBased', splittingArgs=None, setLocationFlag=False):
        """
        _getProcessingACDCSpec_

        Get a ACDC spec for the processing task of a ReReco workload
        """
        if splittingArgs is None:
            splittingArgs = {'lumis_per_job': 8}
        factory = ReRecoWorkloadFactory()
        rerecoArgs = ReRecoWorkloadFactory.getTestArguments()
        rerecoArgs["ConfigCacheID"] = createConfig(rerecoArgs["CouchDBName"])
        rerecoArgs["Requestor"] = self.user
        rerecoArgs["Group"] = self.group
        Tier1ReRecoWorkload = factory.factoryWorkloadConstruction(self.workflowName, rerecoArgs)
        Tier1ReRecoWorkload.truncate('ACDC_%s' % self.workflowName, '/%s/DataProcessing' % self.workflowName,
                                     self.couchUrl, self.acdcDBName)
        Tier1ReRecoWorkload.setJobSplittingParameters('/ACDC_%s/DataProcessing' % self.workflowName, splittingAlgo,
                                                      splittingArgs)
        Tier1ReRecoWorkload.setSiteWhitelist(self.siteWhitelist)
        if setLocationFlag:
            Tier1ReRecoWorkload.setTrustLocationFlag(setLocationFlag)
        #            Tier1ReRecoWorkload.setSiteWhitelist(self.siteWhitelist)
        return Tier1ReRecoWorkload

    def getMergeACDCSpec(self, splittingAlgo='ParentlessMergeBySize', splittingArgs=None):
        """
        _getMergeACDCSpec_

        Get a ACDC spec for the merge task of a ReReco workload
        """
        if splittingArgs is None:
            splittingArgs = {}

        factory = ReRecoWorkloadFactory()
        rerecoArgs = ReRecoWorkloadFactory.getTestArguments()
        rerecoArgs["ConfigCacheID"] = createConfig(rerecoArgs["CouchDBName"])
        rerecoArgs["Requestor"] = self.user
        rerecoArgs["Group"] = self.group
        Tier1ReRecoWorkload = factory.factoryWorkloadConstruction(self.workflowName, rerecoArgs)
        Tier1ReRecoWorkload.truncate('ACDC_%s' % self.workflowName,
                                     '/%s/DataProcessing/DataProcessingMergeRECOoutput' % self.workflowName,
                                     self.couchUrl, self.acdcDBName)
        Tier1ReRecoWorkload.setJobSplittingParameters('/ACDC_%s/DataProcessingMergeRECOoutput' % self.workflowName,
                                                      splittingAlgo, splittingArgs)
        return Tier1ReRecoWorkload

    def stuffACDCDatabase(self, numFiles=50, lumisPerFile=20, lumisPerACDCRecord=2):
        """
        _stuffACDCDatabase_

        Fill the ACDC database with ACDC records, both for processing
        and merge
        """
        filesetName = '/%s/DataProcessing' % self.workflowName

        for i in range(numFiles):
            for j in range(1, lumisPerFile + 1, lumisPerACDCRecord):
                lfn = '/store/data/a/%d' % i
                acdcFile = File(lfn=lfn, size=100, events=250, locations=self.validLocations, merged=1)
                run = Run(i + 1, *list(range(j, min(j + lumisPerACDCRecord, lumisPerFile + 1))))
                acdcFile.addRun(run)
                acdcDoc = {'collection_name': self.workflowName,
                           'collection_type': 'ACDC.CollectionTypes.DataCollection',
                           'files': {lfn: acdcFile},
                           'fileset_name': filesetName}
                self.acdcDB.queue(acdcDoc)

        filesetName = '/%s/DataProcessing/DataProcessingMergeRECOoutput' % self.workflowName
        for i in range(numFiles):
            for j in range(1, lumisPerFile + 1, lumisPerACDCRecord):
                lfn = '/store/unmerged/b/%d' % i
                acdcFile = File(lfn=lfn, size=100, events=250, locations=set([choice(self.validLocations)]), merged=0)
                run = Run(i + 1, *list(range(j, min(j + lumisPerACDCRecord, lumisPerFile + 1))))
                acdcFile.addRun(run)
                acdcDoc = {'collection_name': self.workflowName,
                           'collection_type': 'ACDC.CollectionTypes.DataCollection',
                           'files': {lfn: acdcFile},
                           'fileset_name': filesetName}
                self.acdcDB.queue(acdcDoc)
        self.acdcDB.commit()

        return

    def testEmptyACDC(self):
        """
        _testEmptyACDC_

        Test that the correct exception is rased when trying an ACDC
        without any ACDC documents. Must test a splittingAlgo that
        works in fixedSizeChunks and singleChunk.
        """
        # SingleChunk
        acdcWorkload = self.getProcessingACDCSpec()
        for task in acdcWorkload.taskIterator():
            policy = ResubmitBlock()
            self.assertRaises(WorkQueueNoWorkError, policy, acdcWorkload, task)

        # FixedSizeChunks
        acdcWorkload = self.getProcessingACDCSpec('FileBased', {'files_per_job': 2000})
        for task in acdcWorkload.taskIterator():
            policy = ResubmitBlock()
            self.assertRaises(WorkQueueNoWorkError, policy, acdcWorkload, task)

        return

    def testUnsupportedACDC(self):
        """
        _testUnsupportedACDC_

        Some tasks with certain algorithms won't work if resubmitted,
        catch those
        """
        acdcWorkload = self.getMergeACDCSpec('WMBSMergeBySize', {})
        for task in acdcWorkload.taskIterator():
            policy = ResubmitBlock()
            self.assertRaises(WorkQueueWMSpecError, policy, acdcWorkload, task)

        return

    def testFixedSizeChunksSplit(self):
        """
        _testFixedSizeChunksSplit_

        Test splitting a resubmit block in fixed size chunks of 250, it will
        use a ReReco ACDC of the processing task.
        """
        self.stuffACDCDatabase()
        acdcWorkload = self.getProcessingACDCSpec('FileBased', {'files_per_job': 5})
        acdcWorkload.data.request.priority = 10000
        for task in acdcWorkload.taskIterator():
            policy = ResubmitBlock()
            units, _, _ = policy(acdcWorkload, task)
            self.assertEqual(len(units), 2)
            for unit in units:
                self.assertEqual(len(unit['Inputs']), 1)
                inputBlock = next(iter(unit['Inputs']))
                self.assertEqual(sorted(unit['Inputs'][inputBlock]), sorted(self.PSNs))
                self.assertEqual(10000, unit['Priority'])
                self.assertEqual(50, unit['Jobs'])
                self.assertEqual(acdcWorkload, unit['WMSpec'])
                self.assertEqual(task, unit['Task'])
                self.assertEqual(500, unit['NumberOfLumis'])
                self.assertEqual(250, unit['NumberOfFiles'])
                self.assertEqual(62500, unit['NumberOfEvents'])
                self.assertEqual(unit['ACDC'], {'database': self.acdcDBName,
                                                'fileset': '/%s/DataProcessing' % self.workflowName,
                                                'collection': self.workflowName,
                                                'server': self.couchUrl})
        return

    def testSingleChunksSplit(self):
        """
        _testSingleChunksSplit_

        Test splitting a resubmit block in a single chunk, it will
        use a ReReco ACDC of the merge task and the processing task.
        This is to test two algorithms and that the split is location
        independent.
        """
        self.stuffACDCDatabase()
        acdcWorkload = self.getProcessingACDCSpec('LumiBased', {'lumis_per_job': 10})
        acdcWorkload.data.request.priority = 10000
        for task in acdcWorkload.taskIterator():
            policy = ResubmitBlock()
            units, _, _ = policy(acdcWorkload, task)
            self.assertEqual(len(units), 1)
            for unit in units:
                self.assertEqual(len(unit['Inputs']), 1)
                inputBlock = next(iter(unit['Inputs']))
                self.assertEqual(sorted(unit['Inputs'][inputBlock]), sorted(self.PSNs))
                self.assertEqual(10000, unit['Priority'])
                self.assertEqual(100, unit['Jobs'])
                self.assertEqual(acdcWorkload, unit['WMSpec'])
                self.assertEqual(task, unit['Task'])
                self.assertEqual(1000, unit['NumberOfLumis'])
                self.assertEqual(500, unit['NumberOfFiles'])
                self.assertEqual(125000, unit['NumberOfEvents'])
                self.assertEqual(unit['ACDC'], {'database': self.acdcDBName,
                                                'fileset': '/%s/DataProcessing' % self.workflowName,
                                                'collection': self.workflowName,
                                                'server': self.couchUrl})

    def testMergeACDCSpec(self):
        """
        _testMergeACDCSpec
        Test ACDC spec for the merge task of a ReReco workload
        """
        self.stuffACDCDatabase()
        acdcWorkload = self.getMergeACDCSpec('ParentlessMergeBySize', {})
        acdcWorkload.setSiteWhitelist(["T2_XX_SiteA", "T2_XX_SiteB", "T2_XX_SiteC"])
        acdcWorkload.data.request.priority = 10000
        for task in acdcWorkload.taskIterator():
            policy = ResubmitBlock()
            units, _, _ = policy(acdcWorkload, task)

            self.assertEqual(len(units), 1)
            for unit in units:
                self.assertEqual(len(unit['Inputs']), 1)
                inputBlock = next(iter(unit['Inputs']))
                self.assertEqual(sorted(unit['Inputs'][inputBlock]), sorted(self.PSNs))
                self.assertEqual(10000, unit['Priority'])
                self.assertEqual(500, unit['Jobs'])
                self.assertEqual(acdcWorkload, unit['WMSpec'])
                self.assertEqual(task, unit['Task'])
                self.assertEqual(1000, unit['NumberOfLumis'])
                self.assertEqual(500, unit['NumberOfFiles'])
                self.assertEqual(125000, unit['NumberOfEvents'])
                self.assertEqual(unit['ACDC'], {'database': self.acdcDBName,
                                                'fileset': '/%s/DataProcessing/DataProcessingMergeRECOoutput' % self.workflowName,
                                                'collection': self.workflowName,
                                                'server': self.couchUrl})
        return

    def testLocalQueueACDCSplit(self):
        """
        _testLocalQueueACDCSplit_

        Test that the local queue split (i.e. with something in self.data) is compatible
        with the information, namely the block name, produced by the global split.
        """
        self.stuffACDCDatabase()
        acdcWorkload = self.getProcessingACDCSpec('LumiBased', {'lumis_per_job': 10})
        acdcWorkload.data.request.priority = 10000
        for task in acdcWorkload.taskIterator():
            policy = ResubmitBlock()
            units, _, _ = policy(acdcWorkload, task)
            self.assertEqual(len(units), 1)
            for unit in units:
                self.assertEqual(len(unit['Inputs']), 1)
                inputBlock = next(iter(unit['Inputs']))
                self.assertEqual(sorted(unit['Inputs'][inputBlock]), sorted(self.PSNs))
                self.assertEqual(10000, unit['Priority'])
                self.assertEqual(100, unit['Jobs'])
                self.assertEqual(acdcWorkload, unit['WMSpec'])
                self.assertEqual(task, unit['Task'])
                self.assertEqual(1000, unit['NumberOfLumis'])
                self.assertEqual(500, unit['NumberOfFiles'])
                self.assertEqual(125000, unit['NumberOfEvents'])
                self.assertEqual(unit['ACDC'],
                                 {'database': self.acdcDBName, 'fileset': '/%s/DataProcessing' % self.workflowName,
                                  'collection': self.workflowName, 'server': self.couchUrl})
                globalUnit = unit
            policy = ResubmitBlock()
            units, _, _ = policy(acdcWorkload, task, data=globalUnit['Inputs'])
            self.assertEqual(len(units), 1)
            for unit in units:
                self.assertEqual(len(unit['Inputs']), 1)
                inputBlock = next(iter(unit['Inputs']))
                self.assertEqual(sorted(unit['Inputs'][inputBlock]), sorted(self.PSNs))
                self.assertEqual(10000, unit['Priority'])
                self.assertEqual(100, unit['Jobs'])
                self.assertEqual(acdcWorkload, unit['WMSpec'])
                self.assertEqual(task, unit['Task'])
                self.assertEqual(1000, unit['NumberOfLumis'])
                self.assertEqual(500, unit['NumberOfFiles'])
                self.assertEqual(125000, unit['NumberOfEvents'])
                self.assertEqual(unit['ACDC'],
                                 {'database': self.acdcDBName, 'fileset': '/%s/DataProcessing' % self.workflowName,
                                  'collection': self.workflowName, 'server': self.couchUrl})
        return

    def testSiteWhitelistsLocation(self):
        """
        _testSiteWhitelistsLocation_

        Test splitting a resubmit block changing the block locations
        according to the given site whitelist.
        """
        self.stuffACDCDatabase()
        acdcWorkload = self.getProcessingACDCSpec('FileBased', {'files_per_job': 5},
                                                  setLocationFlag=True)
        acdcWorkload.data.request.priority = 10000
        for task in acdcWorkload.taskIterator():
            policy = ResubmitBlock()
            units, _, _ = policy(acdcWorkload, task)
            self.assertEqual(len(units), 2)
            for unit in units:
                self.assertEqual(len(unit['Inputs']), 1)
                inputBlock = next(iter(unit['Inputs']))
                self.assertEqual(sorted(unit['Inputs'][inputBlock]), sorted(self.siteWhitelist))
                self.assertEqual(10000, unit['Priority'])
                self.assertEqual(50, unit['Jobs'])
                self.assertEqual(acdcWorkload, unit['WMSpec'])
                self.assertEqual(task, unit['Task'])
                self.assertEqual(500, unit['NumberOfLumis'])
                self.assertEqual(250, unit['NumberOfFiles'])
                self.assertEqual(62500, unit['NumberOfEvents'])
                self.assertEqual(unit['ACDC'], {'database': self.acdcDBName,
                                                'fileset': '/%s/DataProcessing' % self.workflowName,
                                                'collection': self.workflowName,
                                                'server': self.couchUrl})
        return


if __name__ == "__main__":
    unittest.main()



#!/usr/bin/env python
# encoding: utf-8
"""
Scram_t.py

Created by Dave Evans on 2012-01-27.
Copyright (c) 2012 evansde77. All rights reserved.
"""

import unittest
import os
import tempfile

from WMQuality.TestInit import TestInit
from Utils.TemporaryEnvironment import tmpEnv
from Utils.PythonVersion import PY3
from WMCore.WMRuntime.Tools.Scram import (Scram, OS_TO_ARCH, ARCH_TO_OS, getSingleScramArch,
                                          isCMSSWSupported, isEnforceGUIDInFileNameSupported)


class Scram_t(unittest.TestCase):
    def setUp(self):
        self.testInit = TestInit(__file__)
        self.testInit.setLogging()
        self.testDir = self.testInit.generateWorkDir()
        self.oldCwd = os.getcwd()
        if PY3:
            self.assertItemsEqual = self.assertCountEqual

    def tearDown(self):
        self.testInit.delWorkDir()

    def testA(self):
        """
        instantiate a Scram instance in test mode.
        """
        try:
            Scram(
                initialise="/bin/date",
                architecture="slc5_amd64_gcc454",
                version="CMSSW_X_Y_Z",
                test=True
            )
        except Exception as ex:
            msg = "Failed to instantiate Scram in test mode:\n %s " % str(ex)
            self.fail(msg)

    def testB(self):
        """
        instantiante a Scram instance in non-test mode
        limited what we can test here since we dont have scram etc in unittest env
        """
        try:
            Scram(
                initialise="/bin/date",
                architecture="slc5_amd64_gcc454",
                version="CMSSW_X_Y_Z"
            )
        except Exception as ex:
            msg = "Failed to instantiate Scram:\n %s " % str(ex)
            self.fail(msg)

    def testC(self):
        """
        test all method calls in test mode

        """
        s = Scram(
            initialise="/bin/date",
            architecture="slc5_amd64_gcc454",
            version="CMSSW_X_Y_Z",
            directory=self.testDir,
            test=True

        )

        try:
            status = s.project()
        except Exception as ex:
            msg = "Error running Scram.project:\n %s" % str(ex)
            self.fail(msg)

        self.assertEqual(status, 0)
        self.assertTrue(os.path.exists(s.projectArea))
        self.assertTrue("project" in s.lastExecuted)
        self.assertTrue("CMSSW_X_Y_Z" in s.lastExecuted)

        try:
            status = s.runtime()
        except Exception as ex:
            msg = "Error running Scram.runtime:\n %s" % str(ex)
            self.fail(msg)

        self.assertEqual(status, 0)
        self.assertTrue("ru -sh" in s.lastExecuted)
        self.assertTrue("TEST_MODE" in s.runtimeEnv)

        comm = "echo \"Hello World\""
        try:
            status = s(comm)
        except Exception as ex:
            msg = "Failed to call Scram object:\n %s" % str(ex)

        self.assertEqual(status, 0)
        self.assertEqual(s.lastExecuted, comm)

    def testArchMap(self):
        self.assertItemsEqual(OS_TO_ARCH['rhel6'], ['slc5', 'slc6'])
        self.assertItemsEqual(OS_TO_ARCH['rhel7'], ['slc7'])
        self.assertItemsEqual(OS_TO_ARCH['rhel8'], ['el8', 'cc8', 'cs8', 'alma8'])
        self.assertItemsEqual(ARCH_TO_OS['slc6'], ['rhel6'])
        self.assertEqual(len(ARCH_TO_OS), 7)
        self.assertItemsEqual(ARCH_TO_OS['slc7'], ['rhel7'])
        self.assertItemsEqual(ARCH_TO_OS['slc7'], ['rhel7'])

    def testScramArchParsing(self):
        """
        Test the various modes of parsing for the scram arch
        """
        try:
            os.chdir(self.testDir)
            with tempfile.NamedTemporaryFile() as tf:
                tf.write(b'GLIDEIN_REQUIRED_OS = "rhel6" \n')
                tf.write(b'Memory = 2048\n')
                tf.flush()
                with tmpEnv(_CONDOR_MACHINE_AD=tf.name):
                    self.assertEqual(getSingleScramArch('slc6_blah_blah'), 'slc6_blah_blah')
                    self.assertEqual(getSingleScramArch('slc5_blah_blah'), 'slc5_blah_blah')
                    self.assertEqual(getSingleScramArch(['slc6_blah_blah', 'slc7_blah_blah']),
                                      'slc6_blah_blah')
                    self.assertEqual(getSingleScramArch(['slc6_blah_blah', 'slc5_blah_blah']),
                                      'slc6_blah_blah')
                    self.assertEqual(getSingleScramArch(['slc7_blah_blah', 'slc8_blah_blah']), None)
            with tempfile.NamedTemporaryFile() as tf:
                tf.write(b'GLIDEIN_REQUIRED_OS = "rhel7" \n')
                tf.write(b'Memory = 2048\n')
                tf.flush()
                with tmpEnv(_CONDOR_MACHINE_AD=tf.name):
                    self.assertEqual(getSingleScramArch('slc6_blah_blah'), 'slc6_blah_blah')
                    self.assertEqual(getSingleScramArch('slc7_blah_blah'), 'slc7_blah_blah')
                    self.assertEqual(getSingleScramArch(['slc6_blah_blah', 'slc7_blah_blah']),
                                      'slc7_blah_blah')
                    self.assertEqual(getSingleScramArch(['slc6_blah_blah', 'slc5_blah_blah']), None)
                    self.assertEqual(getSingleScramArch(['slc7_blah_blah', 'slc8_blah_blah']),
                                      'slc7_blah_blah')
        except Exception:
            raise
        finally:
            os.chdir(self.oldCwd)
        return

    def testCMSSWSupported(self):
        """
        Test the functionality of isCMSSWSupported function
        """
        self.assertFalse(isCMSSWSupported('CMSSW_1_2_3', ''))
        self.assertFalse(isCMSSWSupported(None, 'a'))
        self.assertFalse(isCMSSWSupported('CMSSW_1_2_3', 'CMSSW_2_2_3'))
        self.assertFalse(isCMSSWSupported('CMSSW_1_2_3', 'CMSSW_1_3_3'))
        self.assertFalse(isCMSSWSupported('CMSSW_1_2_3', 'CMSSW_1_2_4'))
        self.assertFalse(isCMSSWSupported('CMSSW_1_2_3_pre1', 'CMSSW_1_2_3'))
        self.assertFalse(isCMSSWSupported('CMSSW_1_2_3', 'CMSSW_1_2_3_pre1'))
        self.assertFalse(isCMSSWSupported('CMSSW_1_2_3_pre1', 'CMSSW_1_2_3_pre2'))
        self.assertFalse(isCMSSWSupported('CMSSW_1_2_3_pre2', 'CMSSW_1_2_3_pre1'))
        self.assertFalse(isCMSSWSupported('CMSSW_7_1_25_patch2', 'CMSSW_7_6_0'))
        self.assertFalse(isCMSSWSupported('CMSSW_7_3_2', 'CMSSW_10_4_0'))

        self.assertTrue(isCMSSWSupported('CMSSW_1_2_3_pre1', 'CMSSW_1_2_3_pre1'))
        self.assertTrue(isCMSSWSupported('CMSSW_1_2_3', 'CMSSW_1_2_3'))
        self.assertTrue(isCMSSWSupported('CMSSW_2_2_3', 'CMSSW_1_2_3'))
        self.assertTrue(isCMSSWSupported('CMSSW_1_3_3', 'CMSSW_1_2_3'))
        self.assertTrue(isCMSSWSupported('CMSSW_1_2_4', 'CMSSW_1_2_3'))

    def testisEnforceGUIDInFileNameSupported(self):
        """
        Test functionality of the `isEnforceGUIDInFileNameSupported` function
        """
        ### invalid input
        self.assertFalse(isEnforceGUIDInFileNameSupported(None))
        self.assertFalse(isEnforceGUIDInFileNameSupported(''))

        ### forever supported
        self.assertTrue(isEnforceGUIDInFileNameSupported('CMSSW_11_0_0'))
        self.assertTrue(isEnforceGUIDInFileNameSupported('CMSSW_11_0_2'))
        self.assertTrue(isEnforceGUIDInFileNameSupported('CMSSW_11_1_0_pre1'))
        self.assertTrue(isEnforceGUIDInFileNameSupported('CMSSW_11_1_0_patch1'))
        self.assertTrue(isEnforceGUIDInFileNameSupported('CMSSW_11_1_1'))

        ### specific releases
        self.assertTrue(isEnforceGUIDInFileNameSupported('CMSSW_10_2_20_UL'))
        self.assertTrue(isEnforceGUIDInFileNameSupported('CMSSW_9_4_16_UL'))
        self.assertTrue(isEnforceGUIDInFileNameSupported('CMSSW_8_0_34_UL'))
        self.assertTrue(isEnforceGUIDInFileNameSupported('CMSSW_7_1_45_patch3'))

        ### minor supported releases
        self.assertTrue(isEnforceGUIDInFileNameSupported('CMSSW_10_6_8'))
        self.assertTrue(isEnforceGUIDInFileNameSupported('CMSSW_10_6_9'))
        self.assertTrue(isEnforceGUIDInFileNameSupported('CMSSW_10_6_8_patch1'))
        self.assertTrue(isEnforceGUIDInFileNameSupported('CMSSW_10_6_9_patch1'))
        self.assertTrue(isEnforceGUIDInFileNameSupported('CMSSW_10_2_20'))
        self.assertTrue(isEnforceGUIDInFileNameSupported('CMSSW_9_4_16'))
        self.assertTrue(isEnforceGUIDInFileNameSupported('CMSSW_9_3_17'))
        self.assertTrue(isEnforceGUIDInFileNameSupported('CMSSW_8_0_34'))

        ### releases not supported
        self.assertFalse(isEnforceGUIDInFileNameSupported('CMSSW_10_6_7'))
        self.assertFalse(isEnforceGUIDInFileNameSupported('CMSSW_10_7_0'))
        self.assertFalse(isEnforceGUIDInFileNameSupported('CMSSW_10_2_19'))
        self.assertFalse(isEnforceGUIDInFileNameSupported('CMSSW_10_3_10'))
        self.assertFalse(isEnforceGUIDInFileNameSupported('CMSSW_5_3_10'))


if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python
"""
_GetMigratedBlocks_

Oracle implementation on PhEDExInjector.GetMigratedBlocks.
"""
from __future__ import division

from WMComponent.RucioInjector.Database.MySQL.GetMigratedBlocks import GetMigratedBlocks as MySQLBase


class GetMigratedBlocks(MySQLBase):
    """
    _GetMigratedBlocks_

    """
    pass



#!/usr/bin/env python
# encoding: utf-8
"""
AutomaticSeeding.py

Created by Dave Evans on 2010-08-30.
Copyright (c) 2010 Fermilab. All rights reserved.

Automatic Seeding, add a flag to the job baggage to tell the runtime scripts to generate seeds on the WN using
the CMSSW RandomSeed Helper
"""

from WMCore.JobSplitting.Generators.GeneratorInterface import GeneratorInterface



class AutomaticSeeding(GeneratorInterface):
    def __init__(self, **options):
        GeneratorInterface.__init__(self, **options)

    def __call__(self, wmbsJob):
        wmbsJob.addBaggageParameter("seeding", self.__class__.__name__)



#!/usr/bin/env python
"""
Unittests for IteratorTools functions
"""

from __future__ import division, print_function

import unittest

from WMCore.ReqMgr.DataStructs.RequestError import InvalidSpecParameterValue
from WMCore.ReqMgr.Utils.Validation import validateOutputDatasets, validate_request_priority


class ValidationTests(unittest.TestCase):
    """
    unittest for ReqMgr Utils Validation functions
    """

    def testValidateOutputDatasets(self):
        """
        Test the validateOutputDatasets function
        """
        dbsUrl = 'https://cmsweb-prod.cern.ch/dbs/prod/global/DBSReader/'

        outputDsets = ['/PD1/AcqEra1-ProcStr1-v1/GEN']
        self.assertIsNone(validateOutputDatasets(outputDsets, dbsUrl))
        outputDsets.append('/PD1/AcqEra1-ProcStr1-v1/GEN-SIM')
        self.assertIsNone(validateOutputDatasets(outputDsets, dbsUrl))
        outputDsets.append('/PD1/AcqEra1-ProcStr1-v1/GEN-SIM-RAW')
        self.assertIsNone(validateOutputDatasets(outputDsets, dbsUrl))

        outputDsets.append('/PD1/AcqEra1-ProcStr1-v1/GEN')
        with self.assertRaises(InvalidSpecParameterValue):
            validateOutputDatasets(outputDsets, dbsUrl)

        outputDsets.remove('/PD1/AcqEra1-ProcStr1-v1/GEN')
        outputDsets.append('/PD1//AOD')
        with self.assertRaises(InvalidSpecParameterValue):
            validateOutputDatasets(outputDsets, dbsUrl)

        outputDsets.remove('/PD1//AOD')
        outputDsets.append('/PD1/None/AOD')
        with self.assertRaises(InvalidSpecParameterValue):
            validateOutputDatasets(outputDsets, dbsUrl)

        outputDsets.remove('/PD1/None/AOD')
        outputDsets.append('/PD1/AcqEra1-ProcStr1-v1/ALAN')
        with self.assertRaises(InvalidSpecParameterValue):
            validateOutputDatasets(outputDsets, dbsUrl)

    def testRequestPriorityValidation(self):
        """
        Test the `validate_request_priority` function, which validates the
        RequestPriority parameter
        :return: nothing, raises an exception if there are problems
        """
        # test valid cases, integer in the range of [0, 1e6]
        for goodPrio in [0, 100, int(1e6 - 1)]:
            reqArgs = {'RequestPriority': goodPrio}
            print(reqArgs)
            validate_request_priority(reqArgs)

        # test invalid ranges
        for badPrio in [-10, 1e6, 1e7]:
            reqArgs = {'RequestPriority': badPrio}
            with self.assertRaises(InvalidSpecParameterValue):
                validate_request_priority(reqArgs)

        # test invalid data types
        for badPrio in ["1234", 1234.35, 1e6, [123]]:
            reqArgs = {'RequestPriority': badPrio}
            with self.assertRaises(InvalidSpecParameterValue):
                validate_request_priority(reqArgs)


if __name__ == '__main__':
    unittest.main()
